@@ -1,13 +1,13 @@
 define([
     'knockout',
-], function (ko) {
+], function(ko) {
     ko.bindingHandlers.hover = {
-        init: function (element, valueAccessor) {
+        init: function(element, valueAccessor) {
             var value = valueAccessor();
             ko.applyBindingsToNode(element, {
                 event: {
-                    mouseenter: function () { value(true) },
-                    mouseleave: function () { value(false) }
+                    mouseenter: function() { value(true); },
+                    mouseleave: function() { value(false); }
                 }
             });
         }
