@@ -1,5 +1,5 @@
 // knockout-sortable 0.15.0 | (c) 2016 Ryan Niemeyer |  http://www.opensource.org/licenses/mit-license
-;(function(factory) {
+(function(factory) {
     if (typeof define === "function" && define.amd) {
         // AMD anonymous module
         define(["knockout", "jquery", "jquery-ui"], factory);
@@ -51,10 +51,10 @@
             result[dataName] = valueAccessor();
         }
 
-        ko.utils.arrayForEach(["afterAdd", "afterRender", "as", "beforeRemove", "includeDestroyed", "templateEngine", "templateOptions", "nodes"], function (option) {
-            if (options.hasOwnProperty(option)) {
+        ko.utils.arrayForEach(["afterAdd", "afterRender", "as", "beforeRemove", "includeDestroyed", "templateEngine", "templateOptions", "nodes"], function(option) {
+            if (Object.prototype.hasOwnProperty.call(options, option)) {
                 result[option] = options[option];
-            } else if (ko.bindingHandlers.sortable.hasOwnProperty(option)) {
+            } else if (Object.prototype.hasOwnProperty.call(ko.bindingHandlers.sortable, option)) {
                 result[option] = ko.bindingHandlers.sortable[option];
             }
         });
@@ -261,7 +261,7 @@
                             }
 
                             //if the strategy option is unset or false, employ the order strategy involving removal and insertion of items
-                            if (!sortable.hasOwnProperty("strategyMove") || sortable.strategyMove === false) {
+                            if (!Object.prototype.hasOwnProperty.call(sortable, "strategyMove") || sortable.strategyMove === false) {
                                 //do the actual move
                                 if (targetIndex >= 0) {
                                     if (sourceParent) {
