@@ -1,9 +1,11 @@
-define(['arches',
+define([
+    'jquery',
+    'arches',
     'models/abstract',
     'models/node',
     'knockout',
     'underscore'
-], function(arches, AbstractModel, NodeModel, ko, _) {
+], function($, arches, AbstractModel, NodeModel, ko, _) {
     return AbstractModel.extend({
         /**
         * A backbone model to manage graph data
@@ -166,7 +168,12 @@ define(['arches',
             return this._doRequest({
                 type: "POST",
                 url: this.url + this.get('graphid') + '/append_branch',
-                data: JSON.stringify({nodeid:node.nodeid, property: property, graphid: branch_graph.get('graphid')})
+                data: JSON.stringify({
+                    nodeid:node.nodeid, 
+                    property: property, 
+                    graphid: branch_graph.get('graphid'),
+                    return_appended_graph: true,
+                })
             }, function(response, status){
                 if (status === 'success' &&  response.responseJSON) {
                     var branchroot = response.responseJSON.root;
@@ -285,7 +292,7 @@ define(['arches',
                         return false;
                     });
                     this.get('nodes')().forEach(function(node) {
-                        found_node = response.responseJSON.nodes.find(function(response_node){
+                        const found_node = response.responseJSON.nodes.find(function(response_node){
                             return response_node.nodeid === node.nodeid;
                         });
                         if (found_node){
@@ -514,9 +521,9 @@ define(['arches',
          */
         constructTree: function(root, nodes, edges, append){
             var nodeMap = {};
-            var root = !!root ? root : this.get('root');
-            var nodes = !!nodes ? nodes : this.get('nodes')();
-            var edges = !!edges ? edges : this.get('edges')();
+            root = !!root ? root : this.get('root');
+            nodes = !!nodes ? nodes : this.get('nodes')();
+            edges = !!edges ? edges : this.get('edges')();
             nodes.forEach(function(node){
                 nodeMap[node.id] = node;
                 if(!ko.isObservable(node.childNodes)){
