@@ -9,9 +9,10 @@ define([
     'require',
     'uuid',
     'utils/dispose',
+    'viewmodels/alert',
     'viewmodels/tile',
     'utils/set-csrf-token'
-], function($, _, ko, koMapping, arches,  CardModel, CardWidgetModel, require, uuid, dispose) {
+], function($, _, ko, koMapping, arches,  CardModel, CardWidgetModel, require, uuid, dispose, AlertViewModel) {
     /**
     * A viewmodel used for generic cards
     *
@@ -77,14 +78,22 @@ define([
         var nodegroups = params.graphModel.get('nodegroups');
         var multiselect = params.multiselect || false;
         var isWritable = params.card.is_writable || false;
+        this.params = params;
         var selection;
         var emptyConstraint = [{
             uniquetoallinstances: false,
             nodes:[],
-            cardid: self.cardid,
-            constraintid:  uuid.generate()
+            cardid: params.card.cardid,
+            constraintid: uuid.generate()
         }];
 
+        let cardConstraints;
+        if (params.card.constraints?.length) {
+            cardConstraints = params.card.constraints;
+        } else {
+            cardConstraints = emptyConstraint;
+        }
+
         var appliedFunctions = params.appliedFunctions;
         var primaryDescriptorFunction = params.primaryDescriptorFunction;
 
@@ -101,7 +110,7 @@ define([
                 widgets: params.cardwidgets,
                 nodes: params.graphModel.get('nodes'),
                 nodegroup: nodegroup,
-                constraints: params.constraints
+                constraints: cardConstraints
             }),
             datatypelookup: params.graphModel.get('datatypelookup'),
         });
@@ -249,7 +258,8 @@ define([
                         perms: perms,
                         permsLiteral: permsLiteral,
                         parentCard: self,
-                        topCards: params.topCards
+                        topCards: params.topCards,
+                        pageVm: params.pageVm,
                     });
                 })
             ),
@@ -310,10 +320,14 @@ define([
                     complete: function() {
                         loading(false);
                         updateDisplayName(params.resourceId, params.displayname);
+                        // update the index on the tiles themselves
+                        self.tiles().forEach((tile, index) => {
+                            tile.sortorder = index;
+                        });
                     }
                 });
             },
-            reorderCards: function() {
+            reorderCards: function(e) {
                 loading(true);
                 var cards = _.map(self.cards(), function(card, i) {
                     card.model.get('sortorder')(i);
@@ -331,6 +345,22 @@ define([
                     url: arches.urls.reorder_cards,
                     complete: function() {
                         loading(false);
+                    },
+                    error: function(response) {
+                        params.pageVm.alert(
+                            new AlertViewModel(
+                                'ep-alert-red',
+                                response.responseJSON.title,
+                                response.responseJSON.message,
+                                null,
+                                function(){}
+                            )
+                        );
+                        const undoSort = (array, sourceIndex, targetIndex) => {
+                            const [movedItem] = array.splice(targetIndex, 1);
+                            array.splice(sourceIndex, 0, movedItem);
+                        };
+                        undoSort(self.cards, e.sourceIndex, e.targetIndex);
                     }
                 });
             },
@@ -415,7 +445,7 @@ define([
             if (selected) this.expanded(true);
         }, this);
         this.expanded.subscribe(function(expanded) {
-            if (expanded && this.parent) this.parent.expanded(true);
+            if (expanded && this.parent?.expanded) this.parent.expanded(true);
         }, this);
 
         this.isChildSelected = ko.computed(function() {
