@@ -1,7 +1,7 @@
 define([
+    'arches',
     'viewmodels/remote-domain-widget',
-    'arches'
-], function (RemoteDomainWidgetViewModel, arches) {
+], function(arches, RemoteDomainWidgetViewModel) {
     /**
     * A viewmodel used for concept widgets
     *
@@ -10,23 +10,24 @@ define([
     *
     * @param  {string} params - a configuration object
     */
+
     var ConceptWidgetViewModel = function(params) {
         var self = this;
 
         params.prepData = function(data) {
-            data.forEach(function (record) {
+            data.forEach(function(record) {
                 if (record.collector) {
                     record.id = undefined;
                 }
-            })
+            });
             return data;
-        }
+        };
 
         RemoteDomainWidgetViewModel.apply(this, [params]);
 
-        var setUrl = function (id) {
+        var setUrl = function(id) {
             if (id) {
-                self.url(arches.urls.dropdown + '?conceptid=' + id)
+                self.url(arches.urls.dropdown + '?conceptid=' + id);
             }
         };
 
