@@ -10,7 +10,7 @@ define(['underscore', 'knockout', 'knockout-mapping', 'viewmodels/report', 'repo
                 var features = [];
                 ko.unwrap(self.tiles).forEach(function(tile) {
                     _.each(tile.data, function(val) {
-                        if ('features' in val) {
+                        if (val?.features) {
                             features = features.concat(koMapping.toJS(val.features));
                         }
                     }, this);
@@ -29,7 +29,7 @@ define(['underscore', 'knockout', 'knockout-mapping', 'viewmodels/report', 'repo
             var count = 0;
             ko.unwrap(self.tiles).forEach(function(tile) {
                 _.each(tile.data, function(val) {
-                    if ('features' in val) {
+                    if (val?.features) {
                         count += 1;
                     }
                 }, this);
