@@ -5,11 +5,13 @@ define([
     'knockout',
     'knockout-mapping',
     'utils/map-popup-provider',
+    'utils/map-configurator',
+    'utils/aria',
     'templates/views/components/map-popup.htm'
-], function($, _, arches, ko, koMapping, mapPopupProvider) {
+], function($, _, arches, ko, koMapping, mapPopupProvider, mapConfigurator, ariaUtils) {
     const viewModel = function(params) {
         var self = this;
-         
+
 
         var geojsonSourceFactory = function() {
             return {
@@ -114,7 +116,7 @@ define([
                 params.overlayConfigs(overlayConfigs);
             }
         });
-        
+
         this.activeBasemap = ko.observable();  // params.basemap is a string, activeBasemap is a map. Cannot initialize from params.
         this.activeBasemap.subscribe(function(basemap) {
             if (ko.isObservable(params.basemap) && params.basemap() !== basemap.name) {
@@ -307,8 +309,11 @@ define([
             this.mapOptions.fitBoundsOptions = params.fitBoundsOptions;
         }
 
-        this.hideSidePanel = function() {
+        this.hideSidePanel = function(focusElement) {
             self.activeTab(undefined);
+            if(focusElement){
+                ariaUtils.shiftFocus(focusElement);
+            }
         };
 
         this.toggleTab = function(tabName) {
@@ -316,6 +321,7 @@ define([
                 self.activeTab(null);
             } else {
                 self.activeTab(tabName);
+                ariaUtils.shiftFocus('#side-panel');
             }
         };
 
@@ -337,8 +343,19 @@ define([
             const popupFeatures = features.map(feature => {
                 var data = feature.properties;
                 var id = data.resourceinstanceid;
-                data.showEditButton = self.canEdit;
-                const descriptionProperties = ['displayname', 'graph_name', 'map_popup'];
+                const userid = ko.unwrap(self.userid);
+                data.showFilterByFeatureButton = !!params.search;
+                data.sendFeatureToMapFilter = mapPopupProvider.sendFeatureToMapFilter.bind(mapPopupProvider);
+                data.showFilterByFeature = mapPopupProvider.showFilterByFeature.bind(mapPopupProvider);
+                const descriptionProperties = ['displayname', 'graph_name', 'map_popup', 'geometries'];
+                const setEditButtonVisibility = function(data)
+                {
+                    const isFeatureEditable = self.canEdit &&
+                    ko.unwrap(data.permissions)?.users_without_edit_perm?.includes(userid) === false ||
+                    ko.unwrap(data.permissions)?.principal_user?.includes(userid) ||
+                    ko.unwrap(data.permissions)?.users_edit?.includes(userid);
+                    data.showEditButton(isFeatureEditable);
+                }
                 if (id) {
                     if (!self.resourceLookup[id]){
                         data = _.defaults(data, {
@@ -346,17 +363,18 @@ define([
                             'displayname': '',
                             'graph_name': '',
                             'map_popup': '',
+                            'geometries': [],
                             'feature': feature,
+                            'showEditButton': ko.observable(false)
                         });
+
                         if (data.permissions) {
                             try {
                                 data.permissions = JSON.parse(ko.unwrap(data.permissions));
                             } catch (err) {
                                 data.permissions = koMapping.toJS(ko.unwrap(data.permissions));
                             }
-                            if (data.permissions.users_without_edit_perm.indexOf(ko.unwrap(self.userid)) > 0) {
-                                data.showEditButton = false;
-                            }
+                            setEditButtonVisibility(data);
                         }
                         descriptionProperties.forEach(prop => data[prop] = ko.observable(data[prop]));
                         data.reportURL = arches.urls.resource_report;
@@ -365,6 +383,8 @@ define([
                         $.get(arches.urls.resource_descriptors + id, function(data) {
                             data.loading = false;
                             descriptionProperties.forEach(prop => self.resourceLookup[id][prop](data[prop]));
+                            self.resourceLookup[id].permissions = data["permissions"];
+                            setEditButtonVisibility(self.resourceLookup[id]);
                         });
                     }
                     self.resourceLookup[id].feature = feature;
@@ -459,9 +479,52 @@ define([
             });
         };
 
+        this.beforeMove = function(e) {
+            e.cancelDrop = (e.sourceParent!==e.targetParent);
+        };
+
+        this.reorderOverlays = function(e) {
+            const mapOrder = ko.observableArray(e.sourceParent());
+            const newOrder = self.createNewOverlayOrder(mapOrder);
+            self.sendNewOverlayOrder(newOrder)
+        };
+
+        this.keyDownHandler = function (context, e) {
+            // reorder list in the front-end by only using keyboard inputs
+            const li = this
+            const moveOverlays = function (direction) {
+                if (self.overlays().includes(li)) {
+                    const index = self.overlays().indexOf(li);
+                    let newIndex = index
+                    if (direction == "up") {
+                        newIndex--
+                    } else if (direction == "down") {
+                        newIndex++
+                    }
+                    if (newIndex != -1 && newIndex != self.overlays().length) {
+                        const newArr = self.overlays()
+                        newArr.splice(newIndex, 0, newArr.splice(index, 1)[0]);
+                        self.overlays(newArr);
+                    }
+                }
+            }
+
+            if (e.ctrlKey) {
+                switch (e.which) {
+                    case 38:
+                        moveOverlays("up");
+                        break;
+                    case 40:
+                        moveOverlays("down");
+                        break;
+                }
+            }
+        };
+
         this.setupMap = function(map) {
             map.on('load', function() {
                 require(['mapbox-gl', 'mapbox-gl-geocoder'], function(MapboxGl, MapboxGeocoder) {
+                    mapConfigurator.preConfig(map);
                     map.addControl(new MapboxGl.NavigationControl(), 'top-left');
                     map.addControl(new MapboxGl.FullscreenControl({
                         container: $(map.getContainer()).closest('.workbench-card-wrapper')[0]
@@ -469,7 +532,7 @@ define([
                     map.addControl(new MapboxGeocoder({
                         accessToken: MapboxGl.accessToken,
                         mapboxgl: MapboxGl,
-                        placeholder: arches.geocoderPlaceHolder,
+                        placeholder: arches.translations.geocoderPlaceHolder,
                         bbox: arches.hexBinBounds
                     }), 'top-right');
 
@@ -524,6 +587,7 @@ define([
                         self.centerY(parseFloat(center.lat));
                     });
 
+                    mapConfigurator.postConfig(map);
                     self.map(map);
                 });
             });
