@@ -1,16 +1,18 @@
 define([
+    'jquery',
     'knockout',
     'underscore',
     'arches',
     'viewmodels/widget',
     'utils/ontology',
+    'utils/resource',
+    'select-woo',
     'views/components/resource-report-abstract',
-], function(ko, _, arches, WidgetViewModel, ontologyUtils) {
-    var resourceLookup = {};
+], function($, ko, _, arches, WidgetViewModel, ontologyUtils, resourceUtils) {
     var graphCache = {};
 
     require(['views/components/related-instance-creator']);
-    
+
     /**
     * A viewmodel used for generic alert messages
     *
@@ -18,11 +20,13 @@ define([
     * @name ResourceInstanceSelectViewModel
     *
     * @param  {object} params
-    * @param  {object} params.node (optional) - if supplied will assume that a node in an editor is being managed and will show 
+    * @param  {object} params.node (optional) - if supplied will assume that a node in an editor is being managed and will show
     * the table of ontologyProperties below the dropdown otherwise the table will be hidden and you will have to populate params.graphids
     * @param  {boolean} params.graphids (optional) - if params.node is not supplied then you need to supply a list of graphids that can be used to get resource instances for the dropdown
     * @param  {boolean} params.multiple - whether to display multiple values in the dropdown/table
-    * @param  {boolean} params.allowInstanceCreation - whether the dropdown will give the user the option to create a new resource instance
+    * @param  {boolean} params.allowInstanceCreation - whether the dropdown will give the user the option to create a new resource instance, defaults to true
+    * @param  {boolean} params.onlyManageResourceIds - whether the value the dropdown manages should just be a resource id or a related resource object that contains the ontology properties as well as the resource id, defaults to false
+    * @param  {boolean} params.displayOntologyTable - true to display the table beneath the dropdown when users select an item, false if that item should be displayed in the dropdown itself, defaults to !!params.node
     * @param  {string} params.searchString (optional) - will limit the search results by the search string (it has to be full URL) and will override node.config.searchString if there is one
     * @param  {function} params.termFilter (optional) - a function to override the default term filter used when typing into the dropdown to search for resources
     * this.termFilter = function(term, data){
@@ -42,31 +46,35 @@ define([
         params.configKeys = ['placeholder', 'defaultResourceInstance'];
         this.preview = arches.graphs.length > 0;
         this.renderContext = params.renderContext;
-        /* 
-            shoehorn logic to piggyback off of search context functionality. 
-            Should be refactored when we get the chance for better component clarity.
-        */ 
-        if (params.renderContext === 'workflow') {
-            self.renderContext = 'search';
-        }
-
-        this.allowInstanceCreation = params.allowInstanceCreation === false ? false : true;
-        if (self.renderContext === 'search') {
-            this.allowInstanceCreation = params.allowInstanceCreation === true ? true : false;
-        }
-        if (!!params.configForm) {
-            this.allowInstanceCreation = false;
-        }
-
+        this.label = params.label;
+        this.relationship = ko.observable();
+        this.allowInstanceCreation = typeof params.allowInstanceCreation === 'undefined' ? true : !!params.allowInstanceCreation;
+        this.onlyManageResourceIds = !!(params?.onlyManageResourceIds);
+        this.displayOntologyTable = params?.displayOntologyTable ?? !!params.node;
         this.multiple = params.multiple || false;
         this.value = params.value || undefined;
         this.selectedItem = params.selectedItem || ko.observable();
         this.rootOntologyClass = '';
         this.graphIsSemantic = false;
         this.resourceTypesToDisplayInDropDown = ko.observableArray(!!params.graphids ? ko.toJS(params.graphids) : []);
-        this.displayOntologyTable = this.renderContext !== 'search' && !!params.node;
         this.graphIds = ko.observableArray();
         this.searchString = params.searchString || ko.unwrap(params.node?.config.searchString);
+        
+        if (!!params.configForm) {
+            this.allowInstanceCreation = false;
+        }
+        
+        /*
+            we should really not be using the "renderContext" property, but rather
+            we should set the values that define how the component operates directly.
+            ie: allowInstanceCreation, or displayOntologyTable
+            the following bits of code are just for backward compatibility
+        */
+        if (params.renderContext === 'search' || params.renderContext === 'workflow') {
+            self.allowInstanceCreation = params.allowInstanceCreation ?? false;
+            self.displayOntologyTable = params.displayOntologyTable ?? false;
+            self.onlyManageResourceIds = params.onlyManageResourceIds ?? true;
+        }
 
         this.waitingForGraphToDownload = ko.computed(function(){
             if (!!params.node && this.resourceTypesToDisplayInDropDown().length > 0){
@@ -92,13 +100,13 @@ define([
                     .then(function(json){
                         self.graphLookup[graphid] = json.graph;
                         self.graphLookupKeys(Object.keys(self.graphLookup));
-                        self.value.valueHasMutated();
+                        self.graphIds.push(json.graph.graphid);
                         return json.graph;
                     });
             }
         };
 
-        // depending on where the widget is being rendered there are several ways to get the ontologyclass 
+        // depending on where the widget is being rendered there are several ways to get the ontologyclass
         if(!!params.node && params.state !== 'display_value'){
             if(!!params.node.graph && !!params.node.graph.get('root')){
                 this.rootOntologyClass = params.node.graph.get('root').ontologyclass();
@@ -122,7 +130,7 @@ define([
                 self.downloadGraph(graphid);
             });
         }
-    
+
         this.resourceInstanceDisplayName = params.form && params.form.displayname ? params.form.displayname() : '';
         this.makeFriendly = ontologyUtils.makeFriendly;
         this.getSelect2ConfigForOntologyProperties = ontologyUtils.getSelect2ConfigForOntologyProperties;
@@ -143,7 +151,7 @@ define([
                 }
             }
         };
-        
+
         WidgetViewModel.apply(this, [params]);
 
         // if a default resource instance is defined, then show them in the ui
@@ -166,7 +174,7 @@ define([
                 ri.inverseOntologyProperty.subscribe(function(){
                     self.defaultResourceInstance(self.value());
                 });
-                ret.push(ri); 
+                ret.push(ri);
             });
             // only set the default values if the tile has never been saved before OR if this is the config form
             if ((this.tile && !this.tile.noDefaults && ko.unwrap(this.tile.tileid) == "" && ret.length > 0) || !!params.configForm) {
@@ -175,16 +183,12 @@ define([
         }
 
         this.displayValue = ko.observable('');
-        
-        //
-        // this.close is only called if newResourceInstance is True and the user 
-        // decides not to add the new resource instance, and closes the window without adding it
-        //
-        this.close = function(){
-            this.newResourceInstance(null);
+
+        this.openReport = function(resourceId) {
+            this.reportResourceId(resourceId);
+            $('#resource-report-panel button').focus();    
         };
         
-        
         this.setValue = function(valueObject) {
             if (self.multiple) {
                 valueObject = [valueObject];
@@ -199,27 +203,8 @@ define([
                 self.defaultResourceInstance(self.value());
             }
         };
-        
-        this.lookupResourceInstanceData = function(resourceid) {
-            if (resourceLookup[resourceid]) {
-                return Promise.resolve(resourceLookup[resourceid]);
-            } else {
-                return window.fetch(arches.urls.search_results + "?id=" + resourceid)
-                    .then(function(response){
-                        if(response.ok) {
-                            return response.json();
-                        }
-                    })
-                    .then(function(json) {
-                        if (json) {
-                            resourceLookup[resourceid] = json["results"]["hits"]["hits"][0];
-                            return resourceLookup[resourceid];
-                        }
-                    });
-            }
-        };
-        
-        if(self.renderContext !== 'search'){
+
+        if(!self.onlyManageResourceIds){
             var updateNameAndOntologyClass = function(values) {
                 var names = [];
                 var value = ko.unwrap(values);
@@ -238,13 +223,13 @@ define([
                             if(!val.iconClass) {
                                 Object.defineProperty(val, 'iconClass', {value: ko.observable()});
                             }
-                            self.lookupResourceInstanceData(ko.unwrap(val.resourceId))
+                            resourceUtils.lookupResourceInstanceData(ko.unwrap(val.resourceId))
                                 .then(function(resourceInstance) {
                                     if (resourceInstance) {
                                         names.push(resourceInstance["_source"].displayname);
                                         self.displayValue(names.join(', '));
                                         val.resourceName(resourceInstance["_source"].displayname);
-                                        val.iconClass(self.graphLookup[resourceInstance["_source"].graph_id]?.iconclass || 'fa fa-question');
+                                        val?.iconClass(self.graphLookup[resourceInstance["_source"].graph_id]?.iconclass || 'fa fa-question');
                                         val.ontologyClass(resourceInstance["_source"].root_ontology_class);
                                     }
                                 });
@@ -252,7 +237,7 @@ define([
                     });
                 }
             };
-    
+
             self.value.subscribe(updateNameAndOntologyClass);
             // Resolve Resource Instance Names from the incoming values
             self.value.valueHasMutated();
@@ -269,7 +254,7 @@ define([
 
         this.makeObject = function(id, esSource){
             var graph = self.graphLookup[esSource.graph_id];
-            var iconClass = graph.iconclass  || 'fa fa-question';
+            var iconClass = graph?.iconclass  || 'fa fa-question';
 
             var ontologyProperty;
             var inverseOntologyProperty;
@@ -279,13 +264,19 @@ define([
                 inverseOntologyProperty = graph.config.inverseOntologyProperty;
 
                 if (self.node && (!ontologyProperty || !inverseOntologyProperty) ) {
+                    self.relationship(!!self.node.ontologyclass());
                     var ontologyProperties = self.node.config.graphs().find(function(nodeConfigGraph) {
                         return nodeConfigGraph.graphid === graph.graphid;
                     });
 
                     if (ontologyProperties) {
-                        ontologyProperty = ontologyProperty || ontologyProperties.ontologyProperty;
-                        inverseOntologyProperty = inverseOntologyProperty || ontologyProperties.inverseOntologyProperty;
+                        if (ontologyProperties.useOntologyRelationship) {
+                            ontologyProperty = ontologyProperty || ontologyProperties.ontologyProperty;
+                            inverseOntologyProperty = inverseOntologyProperty || ontologyProperties.inverseOntologyProperty;
+                        } else {
+                            ontologyProperty = ontologyProperties.relationshipConcept;
+                            inverseOntologyProperty = ontologyProperties.inverseRelationshipConcept;
+                        }
                     }
                 }
             }
@@ -295,7 +286,7 @@ define([
                 "ontologyProperty": ko.observable(ontologyProperty || ""),
                 "inverseOntologyProperty": ko.observable(inverseOntologyProperty || ""),
                 "resourceXresourceId": ""
-            };            
+            };
             Object.defineProperty(ret, 'resourceName', {value: ko.observable(esSource.displayname)});
             Object.defineProperty(ret, 'ontologyClass', {value: ko.observable(esSource.root_ontology_class)});
             Object.defineProperty(ret, 'iconClass', {value: ko.observable(iconClass)});
@@ -307,84 +298,96 @@ define([
                     self.defaultResourceInstance(self.value());
                 });
             }
-            
+
             return ret;
         };
 
 
         this.url = ko.observable(arches.urls.search_results);
-        this.resourceToAdd = ko.observable("");
+        this.resourceToAdd = ko.observable(null);
 
         this.disabled = ko.computed(function() {
             return ko.unwrap(self.waitingForGraphToDownload) || ko.unwrap(params.disabled) || !!ko.unwrap(params.form?.locked);
         });
+
+        // this is a hack to get the dropdown to clear properly
+        // for some reason setting self.resourceToAdd("") didn't work
+        self.select2ele = null;
+        this.clearDropDown = function(){
+            self.select2ele.val(null).trigger('change');
+        };
         
         this.select2Config = {
-            value: self.renderContext === 'search' ? self.value : self.resourceToAdd,
+            value: self.onlyManageResourceIds ? self.value : self.resourceToAdd,
             clickBubble: true,
             disabled: this.disabled,
             multiple: !self.displayOntologyTable ? params.multiple : false,
             placeholder: this.placeholder() || arches.translations.riSelectPlaceholder,
             closeOnSelect: true,
-            allowClear: self.renderContext === 'search' ? true : false,
+            allowClear: self.displayOntologyTable ? false : true,
             onSelect: function(item) {
                 self.selectedItem(item);
-                if (!(self.renderContext === 'search') || self.allowInstanceCreation) {
-                    if (item._source) {
-                        if (self.renderContext === 'search'){
-                            self.value(item._id);
-                        } else {
-                            var ret = self.makeObject(item._id, item._source);
-                            self.setValue(ret);
-                            window.setTimeout(function() {
-                                if(self.displayOntologyTable){
-                                    self.resourceToAdd("");
-                                }
-                            }, 250);    
-                        }
+                if (item._source) {
+                    if (self.onlyManageResourceIds){
+                        self.value(item._id);
                     } else {
-                        // This section is used when creating a new resource Instance
-                        if(!self.preview){
-                            var params = {
-                                graphid: item._id,
-                                complete: ko.observable(false),
-                                resourceid: ko.observable(),
-                                tileid: ko.observable()
-                            };
-                            self.newResourceInstance(params);
-                            var clearNewInstance = function() {
+                        var ret = self.makeObject(item._id, item._source);
+                        self.setValue(ret);
+                        window.setTimeout(function() {
+                            if(self.displayOntologyTable){
+                                self.clearDropDown();
+                            }
+                        }, 250);    
+                    }
+                } else {
+                    // This section is used when creating a new resource Instance
+                    if(!self.preview){
+                        var params = {
+                            graphid: item._id,
+                            complete: ko.observable(false),
+                            resourceid: ko.observable(),
+                            tileid: ko.observable()
+                        };
+                        self.newResourceInstance(params);
+                        self.select2ele.prop("disabled", "disabled");
+
+                        let resourceCreatorPanel = document.querySelector('#resource-creator-panel');
+                        resourceCreatorPanel.addEventListener("transitionend", () => $(resourceCreatorPanel).find('.resource-instance-card-menu-item.selected').focus()); // focus on the resource creator panel for keyboard readers
+                        params.complete.subscribe(function() {
+                            self.select2ele.prop("disabled", "");
+                            if (params.resourceid()) {
+                                window.fetch(arches.urls.search_results + "?id=" + params.resourceid())
+                                    .then(function(response){
+                                        if(response.ok) {
+                                            return response.json();
+                                        }
+                                        throw("error");
+                                    })
+                                    .then(function(json) {
+                                        var item = json.results.hits.hits[0];
+                                        if (self.displayOntologyTable){
+                                            var ret = self.makeObject(params.resourceid(), item._source);
+                                            self.setValue(ret);
+                                        } else {
+                                            var newOption = new Option(item._source.displayname, params.resourceid(), true, true);
+                                            $(self.select2ele).append(newOption);
+                                            self.value(params.resourceid());
+                                        }
+                                    })
+                                    .finally(function(){
+                                        self.newResourceInstance(null);
+                                        if(self.displayOntologyTable){
+                                            window.setTimeout(function() {
+                                                self.clearDropDown();
+                                            }, 250);
+                                        }
+                                    });
+                            } else {
+                                // user decided not to add the resource
                                 self.newResourceInstance(null);
-                                window.setTimeout(function() {
-                                    self.resourceToAdd("");
-                                }, 250);
-                            };
-                            params.complete.subscribe(function() {
-                                if (params.resourceid()) {
-                                    if (self.renderContext === 'search'){
-                                        self.value(params.resourceid());
-                                        clearNewInstance();
-                                    } else {
-                                        window.fetch(arches.urls.search_results + "?id=" + params.resourceid())
-                                            .then(function(response){
-                                                if(response.ok) {
-                                                    return response.json();
-                                                }
-                                                throw("error");
-                                            })
-                                            .then(function(json) {
-                                                var item = json.results.hits.hits[0];
-                                                var ret = self.makeObject(params.resourceid(), item._source);
-                                                self.setValue(ret);
-                                            })
-                                            .finally(function(){
-                                                clearNewInstance();
-                                            });
-                                    }
-                                } else {
-                                    clearNewInstance();
-                                }
-                            });
-                        }
+                                self.clearDropDown();
+                            }
+                        });
                     }
                 }
             },
@@ -394,7 +397,9 @@ define([
                 },
                 dataType: 'json',
                 quietMillis: 250,
-                data: function(term, page) {
+                data: function(requestParams) {
+                    let term = requestParams.term || '';
+                    let page = requestParams.page || 1;
                     //TODO This regex isn't working, but it would nice fix it so that we can do more robust url checking
                     // var expression = /[-a-zA-Z0-9@:%_\+.~#?&//=]{2,256}\.[a-z]{2,4}\b(\/[-a-zA-Z0-9@:%_\+.~#?&//=]*)?/gi;
                     // var regex = new RegExp(expression);
@@ -407,9 +412,9 @@ define([
                         self.url(arches.urls.search_results);
                         var queryString = new URLSearchParams();
                         if (self.searchString) {
-                            const searchUrl = new URL(self.searchString);
+                            const searchUrl = new URL(self.searchString.startsWith("/") ? `${window.location.origin}${self.searchString}` : self.searchString);
                             queryString = new URLSearchParams(searchUrl.search);
-                        } 
+                        }
                         queryString.set('paging-filter', page);
 
                         // merge resource type filters
@@ -443,10 +448,11 @@ define([
                                 queryString.set('term-filter', JSON.stringify(termFilter));
                             }
                         }
+                        queryString.set('sort-results', 'asc')
                         return queryString.toString();
                     }
                 },
-                results: function(data, page) {
+                processResults: function(data) {
                     if (!data['paging-filter'].paginator.has_next && self.allowInstanceCreation) {
                         self.resourceTypesToDisplayInDropDown().forEach(function(graphid) {
                             var graph = self.graphLookup[graphid];
@@ -461,34 +467,47 @@ define([
                             }
                         });
                     }
+                    data.results.hits.hits.forEach(function(hit){
+                        hit.id = hit._id;
+                    });
                     return {
-                        results: data.results.hits.hits,
-                        more: data['paging-filter'].paginator.has_next
+                        "results": data.results.hits.hits,
+                        "pagination": {
+                            "more": data['paging-filter'].paginator.has_next
+                        }
                     };
                 }
             },
-            id: function(item) {
-                return item._id;
-            },
-            formatResult: function(item) {
-                if (item._source) {
-                    const iconClass = self.graphLookup[item._source.graph_id]?.iconclass;
-                    return `<i class="fa ${iconClass} sm-icon-wrap"></i> ${item._source.displayname}`;
-                } else {
-                    if (self.allowInstanceCreation) {
-                        return '<b> ' + arches.translations.riSelectCreateNew.replace('${graphName}', item.name) + ' . . . </b>';
+            templateResult: function(item) {
+                let res = '';
+                if(!item.loading){
+                    if (item._source) {
+                        const iconClass = self.graphLookup[item._source.graph_id]?.iconclass;
+                        res = `<span style="cursor:pointer"><i class="fa ${iconClass} sm-icon-wrap"></i> ${item._source.displayname}</span>`;
+                    } else {
+                        const graph = self.graphLookup[item._id];
+                        if (self.allowInstanceCreation && graph.publication_id) {
+                            res = '<b> ' + arches.translations.riSelectCreateNew.replace('${graphName}', item.name) + ' . . . </b>';
+                        }
                     }
                 }
+                return $(res);
             },
-            formatSelection: function(item) {
+            templateSelection: function(item) {
+                let ret = '';
                 if (item._source) {
-                    return `<i class="fa ${item._source.iconclass} sm-icon-wrap"></i> ${item._source.displayname}`;
+                    var graph = self.graphLookup[item._source.graph_id];
+                    var iconClass = graph?.iconclass || '';
+                    ret = `<span><i class="fa ${iconClass} sm-icon-wrap"></i> ${item._source.displayname}</span>`;
+                    return $(ret);
                 } else {
-                    return item.name;
+                    ret = item.name ?? item.text;
+                    return ret;
                 }
             },
             initSelection: function(ele, callback) {
-                if(self.renderContext === "search" && self.value() !== "" && !self.graphIds().includes(self.value())) {
+                self.select2ele = ele;
+                if(!self.displayOntologyTable && !!self.value() && !self.graphIds().includes(self.value())) {
                     var values = self.value();
                     if(!Array.isArray(self.value())){
                         values = [self.value()];
@@ -505,26 +524,29 @@ define([
                             resourceId = ko.unwrap(val.resourceId);
                         }
 
-                        var resourceInstance = self.lookupResourceInstanceData(resourceId).then(
+                        var resourceInstance = resourceUtils.lookupResourceInstanceData(resourceId).then(
                             function(resourceInstance) { return resourceInstance; }
                         );
-           
+
                         if (resourceInstance) { lookups.push(resourceInstance); }
                     });
-
                     Promise.all(lookups).then(function(arr){
+                        var ret = [];
                         if (arr.length) {
-                            var ret = arr.map(function(item) {
+                            ret = arr.map(function(item) {
                                 return {"_source":{"displayname": item["_source"].displayname, "iconclass": self.graphLookup[item._source.graph_id]?.iconclass || 'fa fa-question'}, "_id":item["_id"]};
                             });
                             if(self.multiple === false) {
                                 ret = ret[0];
                             }
-                            callback(ret);
                         }
+                        callback(ret);
                     });
                 } else if (self.graphIds().includes(self.value())){
                     self.value(null);
+                    callback([]);
+                } else {
+                    callback([]);
                 }
             }
         };
