@@ -2,19 +2,21 @@ define([
     'knockout',
     'knockout-mapping',
     'underscore',
+    'arches',
     'dropzone',
     'uuid',
     'viewmodels/card-component',
     'views/components/workbench',
     'viewmodels/photo-gallery',
     'templates/views/components/cards/photo-gallery-card.htm',
+    'viewmodels/alert',
     'bindings/slide',
     'bindings/fadeVisible',
     'bindings/dropzone',
     'bindings/gallery',
-], function(ko, koMapping, _, Dropzone, uuid, CardComponentViewModel, WorkbenchComponentViewModel, PhotoGallery, photoGalleryCardTemplate) {
+], function(ko, koMapping, _, arches, Dropzone, uuid, CardComponentViewModel, WorkbenchComponentViewModel, PhotoGallery, photoGalleryCardTemplate, AlertViewModel) {
     const viewModel = function(params) {
-         
+
         params.configKeys = ['acceptedFiles', 'maxFilesize'];
         var self = this;
         CardComponentViewModel.apply(this, [params]);
@@ -48,13 +50,33 @@ define([
 
         this.fileListNodeId = getfileListNode();
 
+        this.maxFilesize = ko.computed(function(){
+            var mfs = "Missing maxFilesize";
+            self.card.widgets().forEach(function(widget){
+                if (widget.node_id() === self.fileListNodeId) {
+                    mfs = widget.config.maxFilesize() || "--";
+                }
+            });
+            return mfs;
+        });
+
+        this.acceptedFiles = ko.computed(function(){
+            return self.card.widgets().find(widget=>widget.node_id() === self.fileListNodeId)?.config.acceptedFiles() || arches.translations.allFormatsAccepted;
+        });
+
+        this.cleanUrl = function(url) {
+            const httpRegex = /^https?:\/\//;
+            return !url || httpRegex.test(url) || url.startsWith(arches.urls.url_subpath) ? url :
+                (arches.urls.url_subpath + url).replace('//', '/');
+        };
+
         this.getUrl = function(tile){
             var url = '';
             var name = '';
             var val = ko.unwrap(tile.data[this.fileListNodeId]);
             if (val && val.length == 1) {
                 {
-                    url = ko.unwrap(val[0].url) || ko.unwrap(val[0].content);
+                    url = self.cleanUrl(ko.unwrap(val[0].url)) || ko.unwrap(val[0].content);
                     name = ko.unwrap(val[0].name);
                 }
             }
@@ -133,31 +155,58 @@ define([
         }
 
         this.addTile = function(file){
-            var newtile;
-            newtile = self.card.getNewTile();
-            var tilevalue = {
-                name: file.name,
-                accepted: true,
-                height: file.height,
-                lastModified: file.lastModified,
-                size: file.size,
-                status: file.status,
-                type: file.type,
-                width: file.width,
-                url: null,
-                file_id: null,
-                index: 0,
-                content: window.URL.createObjectURL(file),
-                error: file.error
-            };
-            newtile.data[self.fileListNodeId]([tilevalue]);
-            newtile.formData.append('file-list_' + self.fileListNodeId, file, file.name);
-            newtile.resourceinstance_id = self.card.resourceinstanceid;
-            if (self.card.tiles().length === 0) {
-                sleep(50);
+            var acceptedFileFormats;
+            var loadFile;
+            acceptedFileFormats = ((ko.unwrap(self.acceptedFiles)).split(',').map(item=>item.trim())).map(format => format.replace('.', ''));
+            if(ko.unwrap(self.acceptedFiles) != arches.translations.allFormatsAccepted && acceptedFileFormats !== undefined && acceptedFileFormats.length > 0){
+                var fileType = file.name.split('.').pop().toLowerCase();
+                if(acceptedFileFormats.includes(fileType)){
+                    loadFile = true;
+                }
+                else{
+                    loadFile = false;
+                }
+            }
+            else{
+                loadFile = true;
+            }
+
+            if (loadFile === true) {
+                var newtile;
+                newtile = self.card.getNewTile();
+                var tilevalue = {
+                    name: file.name,
+                    accepted: true,
+                    height: file.height,
+                    lastModified: file.lastModified,
+                    size: file.size,
+                    status: file.status,
+                    type: file.type,
+                    width: file.width,
+                    url: null,
+                    file_id: null,
+                    index: 0,
+                    content: window.URL.createObjectURL(file),
+                    error: file.error
+                };
+                newtile.data[self.fileListNodeId]([tilevalue]);
+                newtile.formData.append('file-list_' + self.fileListNodeId, file, file.name);
+                newtile.resourceinstance_id = self.card.resourceinstanceid;
+                if (self.card.tiles().length === 0) {
+                    sleep(50);
+                }
+                newtile.save();
+                self.card.newTile = undefined;
+            }
+            else{
+                params.pageVm.alert(new AlertViewModel(
+                    'ep-alert-red',
+                    arches.translations.incorrectFileFormat,
+                    arches.translations.fileFormatNotAccepted(ko.unwrap(self.acceptedFiles)),
+                    null,
+                    function(){}
+                ));
             }
-            newtile.save();
-            self.card.newTile = undefined;
         };
 
         this.dropzoneOptions = {
