@@ -4,7 +4,8 @@ define([
     'knockout', 
     'arches', 
     'viewmodels/concept-select', 
-    'templates/views/components/datatypes/concept.htm', 
+    'templates/views/components/datatypes/concept.htm',
+    'bindings/key-events-click', 
 ], function($, _, ko, arches, ConceptSelectViewModel, conceptDatatypeTemplate) {
     var name = 'concept-datatype-config';
     const viewModel = function(params) {
@@ -12,10 +13,12 @@ define([
         this.search = params.search;
         if (this.search) {
             var filter = params.filterValue();
-            params.config = ko.observable({options:[]});
+            params.config = ko.observable({
+                options:[],
+                placeholder: arches.translations.selectAnOption
+            });
              
-            this.op = ko.observable(filter.op || '');
-            this.placeholder = ko.observable('Select a concept');
+            this.op = ko.observable(filter.op || 'eq');
             this.multiple = ko.observable(false);
             this.searchValue = ko.observable(filter.val || '');
             this.node = params.node;
