@@ -9,7 +9,8 @@ define([
          
         if (this.search) {
             var filter = params.filterValue();
-            this.op = ko.observable(filter.op || '');
+            this.node = params.node;
+            this.op = ko.observable(filter.op || 'eq');
             this.searchValue = ko.observable(filter.val || '');
             this.filterValue = ko.computed(function() {
                 return {
