@@ -11,14 +11,14 @@ define([
         if (this.search) {
             var filter = params.filterValue();
             this.op = ko.observable(filter.op || '~');
-            this.languages = ko.observableArray();
-            this.languages(arches.languages);
-            this.language = ko.observable();
+            this.node = params.node;
+            this.languages = ko.observableArray(arches.languages);
+            this.language = ko.observable(arches.activeLanguage);
             this.searchValue = ko.observable(filter.val || '');
             this.filterValue = ko.computed(function() {
                 return {
                     op: self.op(),
-                    lang: self.language()?.code,
+                    lang: self.language(),
                     val: self.searchValue()
                 };
             }).extend({ throttle: 750 });
