@@ -1,5 +1,5 @@
 define(
-    ['knockout', 'arches', 'geocoder-templates'],
+    ['knockout', 'arches'],
     function(ko, arches) {
         /**
          * A viewmodel used for geocoders
@@ -9,6 +9,7 @@ define(
          *
          * @param  {string} params - a configuration object
          */
+
         var BaseGeocoderViewModel = function(params) {
             var self = this;
             let mapboxgl;
@@ -77,7 +78,7 @@ define(
                     }
                 }
                 return true;
-            }
+            };
 
             /**
              * Reloads the geocode layer when a new geocode request is made
@@ -105,10 +106,10 @@ define(
                 } else {
                     self.map.removeLayer('geocode-point');
                 }
-            }
+            };
 
             this.selection.subscribe(this.updateSelection);
-        }
+        };
 
         return BaseGeocoderViewModel;
     }
