@@ -7,12 +7,14 @@ define([
     'views/components/workbench',
     'templates/views/components/iiif-popup.htm',
     'templates/views/components/iiif-viewer.htm',
+    'select-woo-src/utils',
+    'select-woo-src/data/array',
     'leaflet-iiif',
     'leaflet-fullscreen',
     'leaflet-side-by-side',
     'bindings/select2-query',
     'bindings/leaflet'
-], function($, ko, koMapping, L, arches, WorkbenchViewmodel, iiifPopup, iiifViewerTemplate) {
+], function($, ko, koMapping, L, arches, WorkbenchViewmodel, iiifPopup, iiifViewerTemplate, selectWooUtils, selectWooArrayAdapter) {
     var IIIFViewerViewmodel = function(params) {
         var self = this;
         var abortFetchManifest;
@@ -61,11 +63,14 @@ define([
         const layers = [];
         const secondaryLayers = [];
         const cachedAnnotations = {};
+        this.origCanvasLabel = ko.observable();
 
         this.selectPrimaryPanel.subscribe((value) => {
             // if true, primary panel is being selected
             if(value){
                 this.imageToolSelector(this.canvas());
+                self.origCanvasLabel(self.canvasObject().label);
+                self.canvasLabel(self.canvasObject().label);
                 // preserve state of secondary filters, if secondaryCanvas is set
                 if(self.secondaryCanvas()) {
                     secondaryPanelFilters = self.canvasFilterObject();
@@ -79,6 +84,8 @@ define([
             } else {
                 this.imageToolSelector(this.secondaryCanvas());
                 primaryPanelFilters = self.canvasFilterObject();
+                self.origCanvasLabel(self.secondaryCanvasObject()?.label);
+                self.canvasLabel(self.secondaryCanvasObject()?.label);
                 if(secondaryPanelFilters) {
                     self.brightness(secondaryPanelFilters.brightness);
                     self.saturation(secondaryPanelFilters.saturation);
@@ -136,6 +143,11 @@ define([
             }
         });
 
+        this.showLogo = ko.pureComputed(() => {
+            const imageExtenstion = ["bmp", "gif", "jpeg", "jpg", "png", "svg", "tif", "tiff", "webp"]
+            return !!imageExtenstion.find((ext) => self.manifestLogo().endsWith(ext))
+        });
+
         this.buildAnnotationNodes = params.buildAnnotationNodes || function(json) {
             self.annotationNodes(
                 json.map((node) => {
@@ -284,6 +296,8 @@ define([
                             canvas.thumbnail = canvas.thumbnail["@id"];
                         else if (canvas.images && canvas.images[0] && canvas.images[0].resource)
                             canvas.thumbnail = canvas.images[0].resource["@id"];
+                        canvas.id = self.getCanvasService(canvas);
+                        canvas.text = canvas.label;
                         canvases.push(canvas);
                     });
                 }
@@ -301,13 +315,16 @@ define([
             value: this.manifest,
             clickBubble: true,
             multiple: false,
-            closeOnSelect: false,
+            closeOnSelect: true,
             allowClear: true,
+            placeholder: arches.translations.selectAManifest,
             ajax: {
                 url: arches.urls.iiifmanifest,
                 dataType: 'json',
                 quietMillis: 250,
-                data: function(term, page) {
+                data: function(requestParams) {
+                    let term = requestParams.term || '';
+                    let page = requestParams.page || 1;
                     var data = {
                         start: (page-1)*limit,
                         limit: limit
@@ -316,68 +333,62 @@ define([
                     if (term) data.query = term;
                     return data;
                 },
-                results: function(data, page) {
+                processResults: function(data) {
                     var results = data.results;
                     if (validateUrl(queryTerm)) results.unshift({
                         url: queryTerm,
                         label: queryTerm
                     });
+                    results.forEach((item) => {
+                        item.id = item.url;
+                    });
                     return {
-                        results: results,
-                        more: data.count >= (page*limit)
+                        "results": results,
+                        "pagination": {
+                            "more": data.more
+                        }
                     };
                 }
             },
-            id: function(item) {
-                return item.url;
-            },
-            formatResult: function(item) {
+            templateResult: function(item) {
                 return item.label;
             },
-            formatSelection: function(item) {
+            templateSelection: function(item) {
                 return item.label;
-            },
-            clear: function() {
-                self.manifest('');
-            },
-            isEmpty: ko.computed(function() {
-                return self.manifest() === '' || !self.manifest();
-            }, this),
-            initSelection: function() {
-                return;
             }
         };
 
+        var CustomDataAdapterClass = function(){
+            return {};
+        };
+
+        var CustomDataAdapter = selectWooUtils.Decorate(selectWooArrayAdapter, CustomDataAdapterClass);
+        CustomDataAdapter.prototype.current = function(callback){
+            const canvasObj = self.canvases().find(canvas => self.getCanvasService(canvas) == this.options.options.value());
+            callback([canvasObj]);
+        };
+        CustomDataAdapter.prototype.query = function(params, callback){
+            // self.canvases.subscribe(function(canvases){
+            //     callback({"results": canvases});
+            // });
+            callback({"results": self.canvases()});
+        };
+
         const splitSelectConfig = {
             clickBubble: true,
             multiple: false,
             closeOnSelect: true,
-            allowClear: true,
-            data: () => {
-                const results = this.canvases();
-                return { results };
-            },
-            id: function(item) {
-                return self.getCanvasService(item);
-            },
-            containerCssClass: "split-controls-drop",
+            allowClear: false,
+            dataAdapter: CustomDataAdapter,
             dropdownCssClass: "split-controls-drop",
-            dropdownAutoWidth: true,
-            formatResult: function(item) {
-                return `<div class="image"><img src="${item.thumbnail}"/></div><div class="title">${item.label}</div>`; 
+            templateResult: function(item) {
+                if(item.loading){
+                    return "";
+                }
+                return $(`<div class="image"><img src="${item.thumbnail}" height="50"/></div><div class="title">${item.label}</div>`); 
             },
-            formatSelection: function(item) {
+            templateSelection: function(item) {
                 return item?.label;
-            },
-            clear: function(abc) {
-                self.canvases('');
-            },
-            isEmpty: ko.computed(function() {
-                return self.canvases() === '' || !self.canvases();
-            }, this),
-            initSelection: function(element, callback) {
-                const canvasObj = self.canvases().find(canvas => self.getCanvasService(canvas) == element.val());
-                callback(canvasObj);
             }
         };
 
@@ -390,7 +401,7 @@ define([
             ...splitSelectConfig,
             value: this.canvas
         };
-
+        
         this.imageToolConfig = {
             ...splitSelectConfig,
             value: this.imageToolSelector
@@ -499,9 +510,6 @@ define([
             return { brightness, contrast, saturation, greyscale };
         });
 
-        this.canvasFilter.subscribe((value) => {
-            console.log(value);
-        });
         var updateCanvasLayerFilter = function() {
             var filter = self.canvasFilter();
             var map = self.map();
@@ -682,6 +690,7 @@ define([
                 self.secondaryCanvasObject(canvas);
                 self.canvasLabel(self.getManifestDataValue(canvas, 'label', true));
             }
+            self.origCanvasLabel(self.canvasLabel());
         };
 
         this.canvasClick = function(canvas) {
