@@ -5,7 +5,7 @@ define([
     'viewmodels/map',
     'templates/views/components/map.htm',
     'bindings/mapbox-gl',
-    'bindings/sortable'
+    'bindings/sortable',
 ], function($, _, ko, MapViewModel, mapTemplate) {
     ko.components.register('arches-map', {
         viewModel: MapViewModel,
