@@ -22,6 +22,8 @@ define([
         this.loadedResources = params.loaded_resources;
         this.link = params.link;
         this.message = params.message;
+        this.files = params.files;
+        this.translations = arches.translations;
 
         this.dismiss = function(parent) {
             $.ajax({
