@@ -16,11 +16,11 @@ define([
             this.loading(true);
             this.selectedModule = ko.observable();
             this.activeTab = ko.observable();
-            this.isImport = ko.observable(true);
+            this.activeModules = ko.observable('import');
             this.loadEvents = ko.observable();
             this.selectedLoadEvent = ko.observable();
             this.validated = ko.observable();
-            this.validationError = ko.observableArray();
+            this.validationErrors = ko.observableArray();
             this.paginator = ko.observable();
 
             this.selectedLoadEvent.subscribe(function(val){
@@ -28,7 +28,7 @@ define([
                     self.selectedModule(val.etl_module);
                     self.fetchValidation(val.loadid);
                 } else {
-                    if (self.loadEvents().length) {
+                    if (self.loadEvents() && self.loadEvents().length) {
                         self.selectedLoadEvent(self.loadEvents()[0]);
                     }
                 }
@@ -53,7 +53,11 @@ define([
 
             this.activeTab.subscribe(val => {
                 if (val == "import") {
-                    self.fetchLoadEvent();
+                    self.loadEvents(null);
+                    self.selectedLoadEvent(null);
+                    setTimeout(function() {
+                        self.fetchLoadEvent(1);
+                    }, 300);
                 }
             });
 
@@ -68,6 +72,10 @@ define([
                             return response.json();
                         }
                     }).then(function(data){
+                        data.events.map((event)=> {
+                            event.loading = ko.observable(false);
+                        });
+                        
                         self.loadEvents(data.events);
                         self.paginator(data.paginator);
                         const newSelectedEventData = data.events.find(item => item.loadid === self.selectedLoadEvent().loadid);
@@ -79,9 +87,11 @@ define([
             };
 
             this.loadEvents.subscribe(function(loadEvents) {
-                const loadEventIds = loadEvents.map(loadEvent => loadEvent.loadid);
-                if (!loadEventIds.includes(self.selectedLoadEvent()?.loadid)) {
-                    self.selectedLoadEvent(loadEvents[0]);
+                if (loadEvents) {
+                    const loadEventIds = loadEvents.map(loadEvent => loadEvent.loadid);
+                    if (!loadEventIds.includes(self.selectedLoadEvent()?.loadid)) {
+                        self.selectedLoadEvent(loadEvents[0]);
+                    }    
                 }
             });
 
@@ -108,6 +118,7 @@ define([
                 this.alert(new AlertViewModel('ep-alert-red', undoAlertTitle, undoAlertMessage, function() {
                     return;
                 }, function() {
+                    event.loading(true);
                     const formData = new FormData();
                     const url = arches.urls.etl_manager;
                     event.status = 'reversing';
@@ -124,20 +135,13 @@ define([
                     }).then(function(response) {
                         return response.json();
                     }).then(function() {
-                        //pass
+                        self.fetchLoadEvent();
+                        event.loading(false);
                     });
                 }
                 ));
             };
 
-            this.formatUserName = function(event){
-                if (event.first_name || event.last_name) {
-                    return [event.first_name, event.last_name].join(" ");
-                } else {
-                    return event.username;
-                }
-            };
-
             this.fetchStagedData = function(loadid){
                 const url = arches.urls.etl_manager + "?action=stagedData&loadid="+loadid;
                 window.fetch(url).then(function(response){
@@ -149,6 +153,17 @@ define([
                 });
             };
 
+            this.stopEtl = function(loadid) {
+                const url = arches.urls.etl_manager + "?action=stop&loadid="+loadid;
+                window.fetch(url).then(function(response){
+                    if(response.ok){
+                        return response.json();
+                    }
+                }).then(function(data){
+                    console.log(data);
+                });
+            };
+
             this.fetchValidation = function(loadid){
                 const url = arches.urls.etl_manager + "?action=validate&loadid="+loadid;
                 window.fetch(url).then(function(response){
@@ -157,7 +172,34 @@ define([
                     }
                 }).then(function(data){
                     self.validated(true);
-                    self.validationError(data.data);
+                    const errors = data.data;
+                    errors.map(error => {
+                        error.showDetails = ko.observable(false);
+                        error.details = ko.observable();
+                        error.nodeAlias = self.selectedLoadEvent()?.load_details?.mapping?.find( (x) => x.field.trim() == error.source.trim())?.node;
+                    });
+                    self.validationErrors(errors);
+                });
+            };
+
+            this.getErrorReport = function(loadid){
+                const url = arches.urls.etl_manager + "?action=errorReport&loadid="+loadid;
+                window.open(url);
+            };
+
+            this.getNodeError = function(loadid, error){
+                const url = `${arches.urls.etl_manager}?action=nodeError&nodeid=${error.nodeid}&error=${error.error}&loadid=${loadid}`;
+                window.fetch(url).then(function(response){
+                    if(response.ok){
+                        return response.json();
+                    }
+                }).then(function(data){
+                    const sampleErrors = data?.data.slice(0,5);
+                    self.validationErrors().map(error => {
+                        if (error.nodeid === data.data[0].nodeid) {
+                            error.details(sampleErrors);
+                        }
+                    });
                 });
             };
 
