@@ -12,7 +12,7 @@ define([
     'geohash',
     'geojson-extent',
     'uuid',
-    'geojsonhint',
+    'geojsonhint'
 ], function($, _, ko, arches, mapFilterTemplate, BaseFilter, MapComponentViewModel, binFeatureCollection, mapStyles, turf, geohash,  geojsonExtent, uuid, geojsonhint) {
     var componentName = 'map-filter';
     const viewModel = BaseFilter.extend({
@@ -20,6 +20,13 @@ define([
             var self = this;
              
             this.dependenciesLoaded = ko.observable(false);
+            this.resultsAutoZoomEnabled = ko.observable(arches.mapFilterAutoZoom);
+            this.mapFitBounds = function(bounds, options, force){
+                this.lastResultsBounds = bounds;
+                if(this.resultsAutoZoomEnabled() || force){
+                    this.map().fitBounds(bounds, options);
+                }
+            }
 
             require(['mapbox-gl', 'mapbox-gl-draw'], (mapbox, mbdraw) => {
                 self.mapboxgl = mapbox;
@@ -142,7 +149,7 @@ define([
                     if(geoJSON.features.length > 0){
                         var extent = geojsonExtent(geoJSON);
                         var bounds = new this.mapboxgl.LngLatBounds(extent);
-                        this.map().fitBounds(bounds, {
+                        this.mapFitBounds(bounds, {
                             padding: parseInt(this.buffer(), 10)
                         });
                     }
@@ -267,6 +274,18 @@ define([
                 }
             }, this);
 
+            this.filterByFeatureGeom = function(feature) {
+                if (feature.geometry.type == 'Point' && this.buffer() == 0) { this.buffer(25); }
+                self.searchGeometries.removeAll();
+                this.draw.deleteAll();
+                this.draw.set({
+                    "type": "FeatureCollection",
+                    "features": [feature]
+                });
+                self.searchGeometries([feature]);
+                self.updateFilter();
+            };
+
             var updateSearchResultPointLayer = function() {
                 var pointSource = self.map().getSource('search-results-points');
                 var agg = ko.unwrap(self.searchAggregations);
@@ -294,7 +313,7 @@ define([
                     var geojsonFC = self.filter.feature_collection();
                     var extent = geojsonExtent(geojsonFC);
                     var bounds = new this.mapboxgl.LngLatBounds(extent);
-                    self.map().fitBounds(bounds, {
+                    self.mapFitBounds(bounds, {
                         padding: self.buffer()
                     });
                 } else {
@@ -330,7 +349,7 @@ define([
                 updateSearchResultPointLayer();
             };
 
-            this.filters[componentName](this);
+            this.searchFilterVms[componentName](this);
             this.map.subscribe(function(){
                 this.setupDraw();
                 this.restoreState();
@@ -498,17 +517,17 @@ define([
             });
             var bounds = new this.mapboxgl.LngLatBounds(geojsonExtent(mapData.geom));
             var maxZoom = ko.unwrap(this.maxZoom);
-            this.map().fitBounds(bounds, {
+            this.mapFitBounds(bounds, {
                 maxZoom: maxZoom > 17 ? 17 : maxZoom
-            });
+            }, true);
         },
 
         updateQuery: function() {
             var self = this;
             var queryObj = this.query();
             if (this.filter.feature_collection().features.length > 0) {
-                if (this.getFilter('term-filter').hasTag(this.type) === false) {
-                    this.getFilter('term-filter').addTag('Map Filter Enabled', this.name, this.filter.inverted);
+                if (this.getFilterByType('term-filter-type').hasTag(this.type) === false) {
+                    this.getFilterByType('term-filter-type').addTag('Map Filter Enabled', this.name, this.filter.inverted);
                 }
                 this.filter.feature_collection().features[0].properties['inverted'] = this.filter.inverted();
                 queryObj[componentName] = ko.toJSON(this.filter.feature_collection());
@@ -545,7 +564,7 @@ define([
             this.bufferUnit = ko.observable(bufferUnit).extend({ deferred: true });
             this.filter.inverted = ko.observable(inverted).extend({ deferred: true });
             if (hasSpatialFilter) {
-                this.getFilter('term-filter').addTag('Map Filter Enabled', this.name, this.filter.inverted);
+                this.getFilterByType('term-filter-type').addTag('Map Filter Enabled', this.name, this.filter.inverted);
             }
             this.updateResults();
             this.pageLoaded = true;
@@ -574,12 +593,16 @@ define([
                 "type": "FeatureCollection",
                 "features": []
             });
-            this.getFilter('term-filter').removeTag('Map Filter Enabled');
+            this.getFilterByType('term-filter-type').removeTag('Map Filter Enabled');
             this.draw.deleteAll();
             this.searchGeometries([]);
         },
 
-        fitToAggregationBounds: function() {
+        zoomToAllFeaturesHandler: function(){
+            this.fitToAggregationBounds(true);
+        },
+
+        fitToAggregationBounds: function(forceFitBounds) {
             var agg = this.searchAggregations();
             var aggBounds;
             if (agg && agg.geo_aggs.bounds.bounds && this.map()) {
@@ -596,12 +619,13 @@ define([
                 ];
                 var maxZoom = ko.unwrap(this.maxZoom);
                 maxZoom = maxZoom > 17 ? 17 : maxZoom;
-                this.map().fitBounds(bounds, {
+                forceFitBounds = forceFitBounds == undefined ? !this.pageLoaded : forceFitBounds == true;
+                this.mapFitBounds(bounds, {
                     padding: 45,
                     maxZoom: maxZoom
-                });
+                }, forceFitBounds);
             }
-        }
+        },  
     });
 
     return ko.components.register(componentName, {
