@@ -2,7 +2,7 @@ define([
     'knockout',
     'viewmodels/concept-widget',
     'templates/views/components/widgets/checkbox.htm',
-    'plugins/knockout-select2',
+    'bindings/key-events-click',
 ], function(ko, ConceptWidgetViewModel, conceptCheckboxTemplate) {
     /**
      * registers a select-widget component for use in forms
