@@ -2,7 +2,7 @@ define([
     'knockout',
     'viewmodels/domain-widget',
     'templates/views/components/widgets/checkbox.htm',
-    'plugins/knockout-select2',
+    'bindings/key-events-click',
 ], function(ko, DomainWidgetViewModel, checkboxTemplate) {
     /**
      * registers a select-widget component for use in forms
