@@ -4,6 +4,7 @@ define([
     'arches',
     'viewmodels/widget',
     'templates/views/components/widgets/edtf.htm',
+    'bindings/key-events-click',
 ], function(ko, _, arches, WidgetViewModel, edtfTemplate) {
     /**
     * registers a edtf-widget component for use in forms
@@ -38,7 +39,7 @@ define([
             });
         };
         
-        this.getEdtf(params.value());
+        this.getEdtf(ko.unwrap(params.value));
         if (params.state !== 'report') {
             params.value.subscribe(val => {
                 self.getEdtf(val);
