@@ -4,6 +4,7 @@ define([
     'underscore', 
     'viewmodels/widget',
     'templates/views/components/widgets/switch.htm',
+    'bindings/key-events-click',
 ], function(arches, ko, _, WidgetViewModel, switchWidgetTemplate) {
     /**
     * knockout components namespace used in arches
@@ -61,6 +62,7 @@ define([
                 }
             }
         };
+
         this.getvalue = this.config().getvalue || ko.computed(function(){
             var result = null;
             if (this.value() === this.on) {
@@ -71,6 +73,16 @@ define([
             return result;
         }, this);
 
+        this.getariavalue = ko.computed(function(){
+            var result = null;
+            if (this.getvalue() === null) {
+                result = "mixed";
+            } else {
+                result = this.getvalue();
+            }
+            return result;
+        }, this);
+
         this.setdefault = this.config().setdefault || function(self){
             if(self.defaultValue() === self.on){
                 self.defaultValue(self.null);
@@ -91,6 +103,16 @@ define([
             return result;
         }, this);
 
+        this.getariadefault = ko.computed(function(){
+            var result = null;
+            if (this.getdefault() === null) {
+                result = "mixed";
+            } else {
+                result = this.getdefault();
+            }
+            return result;
+        }, this);
+
         var defaultValue = ko.unwrap(this.defaultValue);
         if (this.value() === null && this.defaultValue() !== null) {
             this.value(this.defaultValue());
