@@ -9,10 +9,9 @@ define([
     'viewmodels/provisional-tile',
     'viewmodels/alert',
     'uuid',
+    'js-cookie',
     'templates/views/components/workflows/workflow-component-abstract.htm',
-], function(_, $, ko, koMapping, arches, GraphModel, CardViewModel, ProvisionalTileViewModel, AlertViewModel, uuid, workflowComponentAbstractTemplate) {
-    const WORKFLOW_COMPONENT_ABSTRACTS_LABEL = 'workflow-component-abstracts';
-
+], function(_, $, ko, koMapping, arches, GraphModel, CardViewModel, ProvisionalTileViewModel, AlertViewModel, uuid, Cookies, workflowComponentAbstractTemplate) {
     function NonTileBasedComponent() {
         var self = this;
          
@@ -25,10 +24,11 @@ define([
             self.complete(false);
             self.saving(true);
 
-            self.savedData(self.value());
-            
-            self.complete(true);
-            self.saving(false);
+            self.setToWorkflowHistory('value', self.value()).then(() => {
+                self.savedData(self.value());
+                self.complete(true);
+                self.saving(false);
+            });
         };
 
         this.reset = function() {
@@ -284,19 +284,33 @@ define([
 
     function AbstractCardAdapter() {  // CURRENTLY IN DEVLEOPMENT, USE AT YOUR OWN RISK!
         var self = this;
-         
-
         this.cardinality = ko.observable();
 
         this.initialize = function() {
             self.loading(true);
-
+            
             $.getJSON(( arches.urls.api_nodegroup(self.componentData.parameters['nodegroupid']) ), function(nodegroupData) {
                 self.cardinality(nodegroupData.cardinality);
 
-                $.getJSON(( arches.urls.resource + `/${self.componentData.parameters['resourceid']}/tiles?nodeid=${self.componentData.parameters['nodegroupid']}` ), function(data) {
+                const resourceInstanceDataPromise = new Promise(function(resolve, _reject) {
+                    const resourceInstanceId = self.componentData.parameters['resourceid'];
+    
+                    if (resourceInstanceId) {
+                        $.getJSON(
+                            ( arches.urls.resource + `/${resourceInstanceId}/tiles?nodeid=${self.componentData.parameters['nodegroupid']}`),
+                            function(data) {
+                                resolve(data);
+                            }
+                        );
+                    }
+                    else {
+                        resolve(null);
+                    }
+                });
+
+                resourceInstanceDataPromise.then( function(data) {
                     if (self.cardinality() === '1') {
-                        if (data['tiles'].length) {
+                        if (data && data['tiles'].length) {
                             self.componentData.parameters['tileid'] = data['tiles'][0]['tileid'];
                             self.complete(true);
                         }
@@ -304,18 +318,55 @@ define([
                     }
                     else if (self.cardinality() === 'n') {
                         MultipleTileBasedComponent.apply(self);
-                        if (data['tiles'].length) {
+                        if (data && data['tiles'].length) {
                             self.complete(true);
                         }
                         
-                        self.onSaveSuccess = function(_savedData) {  // LEGACY -- DO NOT USE
+                        self.card.subscribe(function(card) {
+                            if (!card.selected()) {
+                                card.selected(card.tiles().length);
+                            }
+                        });
+
+                        self.onSaveSuccess = function(savedData) {  // LEGACY -- DO NOT USE
+                            if (!(savedData instanceof Array)) { savedData = [savedData]; }
+
+                            let previouslySavedData = self.savedData();
+                            if (!previouslySavedData) {
+                                previouslySavedData = [];
+                            }
+
+                            self.savedData(
+                                previouslySavedData.concat(
+                                    savedData.map(function(savedDatum) {
+                                        return {
+                                            tileData: savedDatum._tileData(),
+                                            tileId: savedDatum.tileid,
+                                            nodegroupId: savedDatum.nodegroup_id,
+                                            resourceInstanceId: savedDatum.resourceinstance_id,
+                                        };
+                                    })
+                                )
+                            );
+
+                            self.value(self.savedData());
+
                             self.componentData.parameters.dirty(false);
+                            self.card().getNewTile(true);  // `true` is forceNewTile
+                            self.card().selected(true);
                             self.dirty(false);
                             self.saving(false);
                             self.complete(true);
+
+                            /**
+                             * TODO: this is a hack to get around previous data autofilling forms when creating a new tile in cardinality n cards
+                             * It should be removed when we're able to figure out how to prevent that logic
+                             * */ 
+                            window.location.reload();  
                         };
                     }
                 });
+
             });
         };
 
@@ -482,7 +533,12 @@ define([
             /* force the value of current tile data observables */ 
             Object.keys(tile.data).forEach(function(key) {
                 if (ko.isObservable(tile.data[key])) {
-                    tile.data[key](data.data[key]());
+                    if (ko.isObservable(data.data[key])) {
+                        tile.data[key](data.data[key]());
+                    }
+                    else{
+                        tile.data[key](ko.mapping.toJS(data.data[key]));
+                    }
                 }
             });
         };
@@ -571,10 +627,11 @@ define([
 
     function WorkflowComponentAbstract(params) {
         var self = this;
-         
 
         this.workflowId = params.workflowId;
+        this.workflowName = params.workflowName;
         this.componentData = params.componentData;
+        this.workflowHistory = params.workflowHistory;
         this.alert = params.alert;
         
         this.locked = params.locked;
@@ -589,7 +646,7 @@ define([
         this.saving = ko.observable(false);
 
         this.savedComponentPaths = {};
-        this.value = ko.observable();
+        this.value = ko.observable(null);
 
         this.complete = ko.observable(false);
         this.error = ko.observable();
@@ -605,9 +662,9 @@ define([
             }
         });
 
-        this.savedData = ko.observable();
+        this.savedData = ko.observable(null);
         this.savedData.subscribe(function(savedData) {
-            self.setToLocalStorage('value', savedData);
+            self.setToWorkflowHistory('value', savedData);
         });
 
         this.multiTileUpdated = ko.observable();
@@ -639,8 +696,10 @@ define([
                 self.id(uuid.generate());
             }
 
-            if (self.getFromLocalStorage('value')) {
-                self.savedData( self.getFromLocalStorage('value') );
+            const savedValue = self.getSavedValue();
+            if (savedValue) {
+                self.savedData(savedValue);
+                self.value(savedValue);
                 self.complete(true);
             }
         };
@@ -676,13 +735,22 @@ define([
                 self.componentData['parameters']['renderContext'] = 'workflow';
             }
 
-
-
             if (self.componentData.componentType === 'card') {
+                let previouslySavedValue = self.getSavedValue();
+                let previouslySavedResourceInstanceId;
+
+                if (previouslySavedValue) {
+                    if (!(previouslySavedValue instanceof Array)) { previouslySavedValue = [previouslySavedValue]; }
+    
+                    if (previouslySavedValue[0]['resourceInstanceId']) {
+                        previouslySavedResourceInstanceId = previouslySavedValue[0]['resourceInstanceId'];
+                        params['componentData']['parameters']['resourceid'] =  previouslySavedResourceInstanceId;
+                    }
+                }
+
                 AbstractCardAdapter.apply(self);
             }
 
-
             else if (!self.componentData.tilesManaged || self.componentData.tilesManaged === "none") {
                 NonTileBasedComponent.apply(self);
             }
@@ -694,26 +762,37 @@ define([
             }
         };
 
-        this.setToLocalStorage = function(key, value) {
-            var allComponentsLocalStorageData = JSON.parse(localStorage.getItem(WORKFLOW_COMPONENT_ABSTRACTS_LABEL)) || {};
+        this.setToWorkflowHistory = async function(key, value) {
+            const workflowid = self.workflowId;
+            const workflowname = self.workflowName;
+            
+            const workflowHistory = {
+                workflowid,
+                workflowname,
+                completed: false,
+                componentdata: {
+                    // Django view will patch in this key, keeping existing keys
+                    [self.id()]: {
+                        [key]: value,
+                    },
+                },
+            };
 
-            if (!allComponentsLocalStorageData[self.id()]) {
-                allComponentsLocalStorageData[self.id()] = {};
-            }
+            await fetch(arches.urls.workflow_history + workflowid, {
+                method: 'POST',
+                credentials: 'include',
+                headers: {
+                    "X-CSRFToken": Cookies.get('csrftoken')
+                },
+                body: JSON.stringify(workflowHistory),
+            });
 
-            allComponentsLocalStorageData[self.id()][key] = value ? koMapping.toJSON(value) : value;
-
-            localStorage.setItem(
-                WORKFLOW_COMPONENT_ABSTRACTS_LABEL, 
-                JSON.stringify(allComponentsLocalStorageData)
-            );
         };
 
-        this.getFromLocalStorage = function(key) {
-            var allComponentsLocalStorageData = JSON.parse(localStorage.getItem(WORKFLOW_COMPONENT_ABSTRACTS_LABEL)) || {};
-
-            if (allComponentsLocalStorageData[self.id()] && typeof allComponentsLocalStorageData[self.id()][key] !== "undefined") {
-                return JSON.parse(allComponentsLocalStorageData[self.id()][key]);
+        this.getSavedValue = function() {
+            const savedValue = this.workflowHistory.componentdata?.[self.id()];
+            if (savedValue) {
+                return savedValue['value'];
             }
         };
 
@@ -750,10 +829,16 @@ define([
                 }
             });
 
-            if (self.componentData.tilesManaged === "many"){
-                self.saveMultiTiles();
-            } else {
-                self.save();
+            // only saves updated tiles
+            if (ko.unwrap(self.dirty) || ko.unwrap(self.hasDirtyTiles) || ko.unwrap(self.hasUnsavedData)) {
+                if (self.componentData.tilesManaged === "many"){
+                    self.saveMultiTiles();
+                } else {
+                    self.save();
+                }
+            }
+            else {
+                self.complete(true);
             }
         };
 
