@@ -27,7 +27,6 @@ define([
     var GraphDesignerView = BaseManagerView.extend({
         initialize: function(options) {
             var viewModel = options.viewModel;
-
             viewModel.graphid = ko.observable(data.graphid);
             viewModel.activeTab = ko.observable('graph');
             viewModel.viewState = ko.observable('design');
@@ -35,7 +34,9 @@ define([
             viewModel.graphSettingsVisible = ko.observable(false);
             viewModel.graph = koMapping.fromJS(data['graph']);
             viewModel.ontologies = ko.observable(data['ontologies']);
-            viewModel.ontologyClasses = ko.observable(data['ontologyClasses']);
+            viewModel.ontologyClasses = ko.pureComputed(function(){
+                return data['ontologyClasses'].filter((cls) => cls.ontology_id === viewModel.graph.ontology_id());
+            });
             viewModel.cardComponents = data.cardComponents;
             viewModel.appliedFunctions = ko.observable(data['appliedFunctions']);
             viewModel.activeLanguageDir = ko.observable(arches.activeLanguageDir);
@@ -50,7 +51,7 @@ define([
                 else if (viewModel.graphSettingsViewModel && viewModel.graphSettingsViewModel.dirty()) {
                     shouldShowGraphPublishButtons = false;
                 }
-                else if (viewModel.selectedNode() && viewModel.selectedNode().dirty() && viewModel.selectedNode().istopnode == false) {
+                else if (viewModel.isNodeDirty()) {
                     shouldShowGraphPublishButtons = false;
                 }
                 else if (ko.unwrap(viewModel.cardTree.selection)) {
@@ -68,6 +69,10 @@ define([
             });
             viewModel.primaryDescriptorFunction = ko.observable(data['primaryDescriptorFunction']);
 
+            viewModel.isNodeDirty = ko.pureComputed(function() {
+                return viewModel.selectedNode() && viewModel.selectedNode().dirty() && viewModel.selectedNode().istopnode == false;
+            });
+
             var resources = ko.utils.arrayFilter(viewData.graphs, function(graph) {
                 return graph.isresource;
             });
@@ -277,6 +282,22 @@ define([
             });
 
             viewModel.selectedNode = viewModel.graphModel.get('selectedNode');
+            viewModel.updatedCardinalityData = ko.observable();
+
+            // ctrl+S to save any edited/dirty nodes 
+            var keyListener = function(e) {
+                if (e.ctrlKey && e.key === "s") {
+                    e.preventDefault();
+                    if (viewModel.isNodeDirty() || viewModel.graphSettingsViewModel.dirty()) {
+                        viewModel.saveSelectedNode();
+                    }
+                }
+            };
+            document.addEventListener("keydown", keyListener)
+            // dispose of eventlistener
+            this.dispose = function(){
+                document.removeEventListener("keydown", keyListener);
+            };
 
             viewModel.saveNode = function(node) {
                 if (node) {
@@ -288,7 +309,9 @@ define([
                         else {
                             viewModel.cardTree.updateCards(viewModel.selectedNode().nodeGroupId(), data.responseJSON);
                             viewModel.permissionTree.updateCards(viewModel.selectedNode().nodeGroupId(), data.responseJSON);
+                            viewModel.updatedCardinalityData([data.responseJSON, viewModel.graphSettingsViewModel]);
                         }
+
                         viewModel.loading(false);
                     });
                 }
@@ -304,7 +327,8 @@ define([
                 graph: viewModel.graph,
                 appliedFunctions: viewModel.appliedFunctions,
                 primaryDescriptorFunction: viewModel.primaryDescriptorFunction,
-                graphModel: viewModel.graphModel
+                graphModel: viewModel.graphModel,
+                pageVm: viewModel,
             });
 
             viewModel.permissionTree = new CardTreeViewModel({
@@ -346,7 +370,8 @@ define([
                 node: viewModel.selectedNode,
                 appliedFunctions: viewModel.appliedFunctions,
                 primaryDescriptorFunction: viewModel.primaryDescriptorFunction,
-                restrictedNodegroups: data.restrictedNodegroups
+                restrictedNodegroups: data.restrictedNodegroups,
+                updatedCardinalityData: viewModel.updatedCardinalityData,
             });
 
             viewModel.branchListView = new BranchListView({
@@ -415,7 +440,8 @@ define([
                 permissionTree: viewModel.permissionTree,
                 appliedFunctions: viewModel.appliedFunctions,
                 primaryDescriptorFunction: viewModel.primaryDescriptorFunction,
-                restrictedNodegroups: data.restrictedNodegroups
+                restrictedNodegroups: data.restrictedNodegroups,
+                pageVm: viewModel,
             });
 
             viewModel.graphTree.branchListVisible.subscribe(function(visible) {
@@ -561,7 +587,7 @@ define([
 
             viewModel.activeTab.subscribe(function(tab) {
                 viewModel.helpTemplate(helpContentLookup[tab]['template']);
-                viewModel.getHelp();
+                viewModel.getHelp(viewModel.helpTemplate());
                 switch (tab) {
                 case 'card':
                     updateCardSelection();
