@@ -7,9 +7,10 @@ define([
     'viewmodels/card',
     'models/card-widget',
     'uuid',
+    'viewmodels/alert',
     'bindings/sortable',
     'bindings/scrollTo',
-], function($, _, ko, arches, data, CardViewModel, CardWidgetModel, uuid) {
+], function($, _, ko, arches, data, CardViewModel, CardWidgetModel, uuid, AlertViewModel) {
     var CardTreeViewModel = function(params) {
         var self = this;
         var filter = ko.observable('');
@@ -167,7 +168,7 @@ define([
                 return;
             },
             beforeMove: function(e) {
-                e.cancelDrop = (e.sourceParent!==e.targetParent);
+                e.cancelDrop = (e.sourceParent !== e.targetParent);
             },
             updateCard: function(parents, card, data) {
                 var updatedCards = [];
@@ -310,6 +311,7 @@ define([
                     parentCard: parent,
                     constraints: getBlankConstraint(data.card),
                     topCards: self.topCards,
+                    pageVm: params.pageVm,
                 });
                 parentcards.push(newCardViewModel);
 
@@ -340,7 +342,7 @@ define([
                 self.cachedFlatTree = self.flattenTree(self.topCards(), []);
                 return newCardViewModel;
             },
-            reorderCards: function() {
+            reorderCards: function(e) {
                 loading(true);
                 var cards = _.map(self.topCards(), function(card, i) {
                     card.model.get('sortorder')(i);
@@ -358,6 +360,22 @@ define([
                     url: arches.urls.reorder_cards,
                     complete: function() {
                         loading(false);
+                    },
+                    error: function(response) {
+                        params.pageVm.alert(
+                            new AlertViewModel(
+                                'ep-alert-red',
+                                response.responseJSON.title,
+                                response.responseJSON.message,
+                                null,
+                                function(){}
+                            )
+                        );
+                        const undoSort = (array, sourceIndex, targetIndex) => {
+                            const [movedItem] = array.splice(targetIndex, 1);
+                            array.splice(sourceIndex, 0, movedItem);
+                        };
+                        undoSort(self.topCards, e.sourceIndex, e.targetIndex);
                     }
                 });
             },
@@ -431,7 +449,8 @@ define([
                 userisreviewer: true,
                 perms: ko.observableArray(),
                 permsLiteral: ko.observableArray(),
-                topCards: self.topCards
+                topCards: self.topCards,
+                pageVm: params.pageVm,
             });
         }));
 
