@@ -1,10 +1,10 @@
 define([
     'jquery',
     'underscore',
-    'arches',
     'backbone',
-    'knockout'
-], function($, _, arches, Backbone, ko) {
+    'knockout',
+    'arches',
+], function($, _, Backbone, ko, arches) {
     var PermissionSettingsForm = Backbone.View.extend({
         /**
         * A backbone view representing a card component form
