@@ -9,11 +9,13 @@ define([
     'viewmodels/alert',
     'views/provisional-history-list',
     'views/notifications-list',
+    'utils/aria',
+    'utils/back-to-top',
     'bindings/scrollTo',
     'bootstrap',
     'bindings/slide',
     'jquery-ui',
-], function($, _, Backbone, ko, moment, arches, viewData, AlertViewModel, ProvisionalHistoryList, NotificationsList) {
+], function($, _, Backbone, ko, moment, arches, viewData, AlertViewModel, ProvisionalHistoryList, NotificationsList, ariaUtils, backToTop) {
     /**
     * A backbone view representing a basic page in arches.  It sets up the
     * viewModel defaults, optionally accepts additional view model data and
@@ -43,6 +45,8 @@ define([
             this.viewModel.helploaded = ko.observable(false);
             this.viewModel.helploading = ko.observable(false);
             this.viewModel.helpOpen = ko.observable(false);
+            this.viewModel.editsOpen = ko.observable(false);
+            this.viewModel.notifsOpen = ko.observable(false);
             this.viewModel.provisionalHistoryList = new ProvisionalHistoryList({
                 items: ko.observableArray(),
                 helploading: this.viewModel.helploading
@@ -64,14 +68,17 @@ define([
                 dirty: ko.observable(false),
                 showConfirmNav: ko.observable(false),
                 navDestination: ko.observable(''),
+                handleEscKey: ariaUtils.handleEscKey,
+                shiftFocus: ariaUtils.shiftFocus,
+                backToTopHandler: backToTop.backToTopHandler,
                 urls: arches.urls,
                 navigate: function(url, bypass) {
                     if (!bypass && self.viewModel.dirty()) {
                         self.viewModel.navDestination(url);
                         self.viewModel.alert(new AlertViewModel(
-                            'ep-alert-blue', 
-                            arches.translations.confirmNav.title, 
-                            arches.translations.confirmNav.text, 
+                            'ep-alert-blue',
+                            arches.translations.confirmNav.title,
+                            arches.translations.confirmNav.text,
                             function() {
                                 self.viewModel.showConfirmNav(false);
                             }, function() {
@@ -84,17 +91,18 @@ define([
                     self.viewModel.loading(true);
                     window.location.assign(url);
                 },
-                getHelp: function() {
+                getHelp: function(template) {
                     self.viewModel.helploading(true);
-                    var el = $('.ep-help-content');
+                    var el = document.createElement('div');
+                    $('.ep-help-content').append(el);
                     $.ajax({
                         type: "GET",
                         url: arches.urls.help_template,
-                        data: {'template': self.viewModel.helpTemplate()}
+                        data: {'template': template}
                     }).done(function(data) {
-                        el.html(data);
+                        $(el).html(data);
                         self.viewModel.helploading(false);
-                        $('.ep-help-topic-toggle').click(function() {
+                        $(el).find('.ep-help-topic-toggle').click(function() {
                             var sectionEl = $(this).closest('div');
                             var iconEl = $(this).find('i');
                             if (iconEl.hasClass("fa-chevron-right")) {
@@ -105,9 +113,17 @@ define([
                                 iconEl.addClass("fa-chevron-right");
                             }
                             var contentEl = $(sectionEl).find('.ep-help-topic-content').first();
+                            let contentExpanded = contentEl.css('display');
+                            if (contentExpanded) {
+                                if (contentExpanded === 'none') {
+                                    $(this).attr('aria-expanded', 'true');
+                                } else if (contentExpanded === 'block') {
+                                    $(this).attr('aria-expanded', 'false');
+                                }
+                            }
                             contentEl.slideToggle();
                         });
-                        $('.reloadable-img').click(function(){
+                        $(el).find('.reloadable-img').click(function(){
                             $(this).attr('src', $(this).attr('src'));
                         });
                     });
@@ -117,7 +133,37 @@ define([
                 },
                 getNotifications: function() {
                     self.viewModel.notifsList.updateList();
-                }
+                },
+                openNotifs: function(openButton, escListenScope, closeButton) {
+                    self.viewModel.getNotifications();
+                    self.viewModel.notifsOpen(!(self.viewModel.notifsOpen()));
+                    setTimeout(() => {self.viewModel.handleEscKey(openButton, escListenScope, closeButton)}, 500);
+                },
+                openEdits: function(openButton, escListenScope, closeButton) {
+                    self.viewModel.getProvisionalHistory();
+                    self.viewModel.editsOpen(!(self.viewModel.editsOpen()));
+                    setTimeout(() => {self.viewModel.handleEscKey(openButton, escListenScope, closeButton)}, 500);
+                },
+                openHelp: function(helpTemplates, openButton, escListenScope, closeButton) {
+                    helpTemplates.forEach(template => self.viewModel.getHelp(template));
+                    self.viewModel.helpOpen(!(self.viewModel.helpOpen()));
+                    setTimeout(() => {self.viewModel.handleEscKey(openButton, escListenScope, closeButton)}, 500);
+                },
+                closeNotifs: function() {
+                    self.viewModel.getNotifications();
+                    self.viewModel.notifsOpen(false);
+                    self.viewModel.shiftFocus('#ep-notifs-button');
+                },
+                closeEdits: function() {
+                    self.viewModel.editsOpen(false);
+                    self.viewModel.shiftFocus('#ep-edits-button');
+                },
+                closeHelp: function() {
+                    let el = $('.ep-help-content');
+                    el.empty();
+                    self.viewModel.helpOpen(false);
+                    self.viewModel.shiftFocus('#ep-help-button');
+                },
             });
             self.viewModel.notifsList.items.subscribe(function(list) {
                 self.viewModel.unreadNotifs((list.length > 0));
@@ -129,6 +175,13 @@ define([
                 self.viewModel.loading(true);
             });
 
+            window.addEventListener('pageshow', function(event) {
+                if (event.persisted) {
+                    // If the page was restored from cache, hide the loading mask
+                    self.viewModel.loading(false);
+                }
+            });
+
             Backbone.View.apply(this, arguments);
             return this;
         },
@@ -136,15 +189,10 @@ define([
         initialize: function() {
             ko.applyBindings(this.viewModel);
             this.viewModel.getNotifications();
+
             $('[data-toggle="tooltip"]').tooltip();
 
-            $('.ep-edits-toggle').click(function(){
-                $('#ep-edits-panel').toggle('slide', { direction: 'right' });
-            });
-
-            $('.ep-notifs-toggle').click(function(){
-                $('#ep-notifs-panel').toggle('slide', { direction: 'right' });
-            });
+            backToTop.scrollToTopHandler();
         }
     });
     return PageView;
