@@ -1,11 +1,15 @@
 define([
     'knockout',
     'views/base-manager',
-    'plugin-data'
+    'views/plugin-data',
+    'plugins'
 ], function(ko, BaseManagerView, data) {
     if (!data.config) data.config = {};
+    
     data.config.loading = ko.observable(false);
     data.config.alert = ko.observable(null);
+    data.config.plugin = data;
+
     return new BaseManagerView({
         viewModel: {
             loading: data.config.loading,
