@@ -1,4 +1,4 @@
-define(['jquery', 'backbone', 'select2'], function($, Backbone) {
+define(['jquery', 'backbone', 'select-woo'], function($, Backbone) {
     return Backbone.View.extend({
         initialize: function() {
             this.$el.find('.arches_simple_search').select2({
