@@ -22,6 +22,7 @@ import datetime
 import json
 import pytz
 import logging
+from types import SimpleNamespace
 from django.db import IntegrityError
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.exceptions import ValidationError
@@ -29,7 +30,7 @@ from django.db import transaction
 from django.db.models import Q
 from django.contrib.auth.models import User
 from django.utils import timezone
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 from arches.app.models import models
 from arches.app.models.resource import Resource
 from arches.app.models.resource import EditLog
@@ -106,6 +107,20 @@ class Tile(models.TileModel):
                         tile.parenttile = self
                         self.tiles.append(tile)
 
+        self.serialized_graph = None
+        self.load_serialized_graph()
+
+    def load_serialized_graph(self, raise_if_missing=False):
+        try:
+            resource = self.resourceinstance
+        except models.ResourceInstance.DoesNotExist:
+            return
+        published_graph = resource.graph.get_published_graph(
+            raise_if_missing=raise_if_missing
+        )
+        if published_graph:
+            self.serialized_graph = published_graph.serialized_graph
+
     def save_edit(
         self,
         user={},
@@ -142,8 +157,12 @@ class Tile(models.TileModel):
         edit.nodegroupid = self.nodegroup_id
         edit.tileinstanceid = self.tileid
         if provisional_edit_log_details is not None:
-            edit.provisional_user_username = getattr(provisional_edit_log_details["provisional_editor"], "username", "")
-            edit.provisional_userid = getattr(provisional_edit_log_details["provisional_editor"], "id", "")
+            edit.provisional_user_username = getattr(
+                provisional_edit_log_details["provisional_editor"], "username", ""
+            )
+            edit.provisional_userid = getattr(
+                provisional_edit_log_details["provisional_editor"], "id", ""
+            )
             edit.provisional_edittype = provisional_edit_log_details["action"]
             user = provisional_edit_log_details["user"]
         edit.userid = getattr(user, "id", "")
@@ -151,13 +170,16 @@ class Tile(models.TileModel):
         edit.user_firstname = getattr(user, "first_name", "")
         edit.user_lastname = getattr(user, "last_name", "")
         edit.user_username = getattr(user, "username", "")
-        edit.resourcedisplayname = Resource.objects.get(resourceinstanceid=self.resourceinstance.resourceinstanceid).displayname()
+        edit.resourcedisplayname = Resource.objects.get(
+            resourceinstanceid=self.resourceinstance.resourceinstanceid
+        ).displayname()
         edit.oldvalue = old_value
         edit.newvalue = new_value
         edit.timestamp = timestamp
         edit.edittype = edit_type
         edit.newprovisionalvalue = newprovisionalvalue
         edit.oldprovisionalvalue = oldprovisionalvalue
+        edit.note = note
         if transaction_id is not None:
             edit.transactionid = transaction_id
         edit.save()
@@ -170,7 +192,9 @@ class Tile(models.TileModel):
                 result = False
         return result
 
-    def apply_provisional_edit(self, user, data, action="create", status="review", existing_model=None):
+    def apply_provisional_edit(
+        self, user, data, action="create", status="review", existing_model=None
+    ):
         """
         Creates or updates the json stored in a tile's provisionaledits db_column
 
@@ -178,7 +202,9 @@ class Tile(models.TileModel):
         if self.tile_collects_data() is True and data != {}:
 
             utc_date_format = "%Y-%m-%dT%H:%M:%S.%fZ"
-            timestamp_utc = str(datetime.datetime.now(pytz.utc).strftime(utc_date_format))
+            timestamp_utc = str(
+                datetime.datetime.now(pytz.utc).strftime(utc_date_format)
+            )
 
             provisionaledit = {
                 "value": data,
@@ -190,7 +216,10 @@ class Tile(models.TileModel):
             }
 
             # if this tile has been previously saved and already has provisional edits on it then
-            if existing_model is not None and existing_model.provisionaledits is not None:
+            if (
+                existing_model is not None
+                and existing_model.provisionaledits is not None
+            ):
                 provisionaledits = existing_model.provisionaledits
                 provisionaledits[str(user.id)] = provisionaledit
             else:
@@ -204,9 +233,8 @@ class Tile(models.TileModel):
         been approved by a user in the resource reviewer group
 
         """
-
         result = False
-        if self.provisionaledits is not None and len(self.data) == 0:
+        if self.provisionaledits is not None and not any(self.data.values()):
             result = True
 
         return result
@@ -236,13 +264,14 @@ class Tile(models.TileModel):
             return
         card = models.CardModel.objects.get(nodegroup=self.nodegroup)
         constraints = models.ConstraintModel.objects.filter(card=card)
-        if constraints.count() > 0:
+        if constraints.exists():
             for constraint in constraints:
                 if constraint.uniquetoallinstances is True:
                     tiles = models.TileModel.objects.filter(nodegroup=self.nodegroup)
                 else:
                     tiles = models.TileModel.objects.filter(
-                        Q(resourceinstance_id=self.resourceinstance.resourceinstanceid) & Q(nodegroup=self.nodegroup)
+                        Q(resourceinstance_id=self.resourceinstance.resourceinstanceid)
+                        & Q(nodegroup=self.nodegroup)
                     )
                 nodes = [node for node in constraint.nodes.all()]
                 for tile in tiles:
@@ -267,7 +296,9 @@ class Tile(models.TileModel):
                                         break
                             if datatype.values_match(tile_data, self.data[nodeid]):
                                 match = True
-                                duplicate_values.append(datatype.get_display_value(tile, node))
+                                duplicate_values.append(
+                                    datatype.get_display_value(tile, node)
+                                )
                             else:
                                 match = False
                                 break
@@ -276,7 +307,9 @@ class Tile(models.TileModel):
                                 "This card violates a unique constraint. \
                                 The following value is already saved: "
                             )
-                            raise TileValidationError(message + (", ").join(duplicate_values))
+                            raise TileValidationError(
+                                message + (", ").join(duplicate_values)
+                            )
 
     def check_for_missing_nodes(self):
         if settings.BYPASS_REQUIRED_VALUE_TILE_VALIDATION:
@@ -284,12 +317,28 @@ class Tile(models.TileModel):
         missing_nodes = []
         for nodeid, value in self.data.items():
             try:
-                node = models.Node.objects.get(nodeid=nodeid)
+                try:
+                    node = SimpleNamespace(
+                        **next(
+                            (
+                                x
+                                for x in self.serialized_graph["nodes"]
+                                if x["nodeid"] == nodeid
+                            ),
+                            None,
+                        )
+                    )
+                except:
+                    node = models.Node.objects.get(nodeid=nodeid)
                 datatype = self.datatype_factory.get_instance(node.datatype)
                 datatype.clean(self, nodeid)
                 if self.data[nodeid] is None and node.isrequired is True:
-                    if len(node.cardxnodexwidget_set.all()) > 0:
-                        missing_nodes.append(node.cardxnodexwidget_set.all()[0].label)
+                    if not isinstance(node, models.Node):
+                        node = models.Node.objects.get(nodeid=nodeid)
+
+                    first_card_x_node_x_widget = node.cardxnodexwidget_set.first()
+                    if first_card_x_node_x_widget:
+                        missing_nodes.append(str(first_card_x_node_x_widget.label))
                     else:
                         missing_nodes.append(node.name)
             except Exception:
@@ -315,14 +364,29 @@ class Tile(models.TileModel):
 
         tile_errors = []
         for nodeid, value in self.data.items():
-            node = models.Node.objects.get(nodeid=nodeid)
+            try:
+                node = SimpleNamespace(
+                    **next(
+                        (
+                            x
+                            for x in self.serialized_graph["nodes"]
+                            if x["nodeid"] == nodeid
+                        ),
+                        None,
+                    )
+                )
+                node.pk = uuid.UUID(node.nodeid)
+            except TypeError:  # will catch if serialized_graph is None
+                node = models.Node.objects.get(nodeid=nodeid)
             datatype = self.datatype_factory.get_instance(node.datatype)
             error = datatype.validate(value, node=node, strict=strict, request=request)
             tile_errors += error
             for error_instance in error:
                 if error_instance["type"] == "ERROR":
                     if raise_early:
-                        raise TileValidationError(_("{0}".format(error_instance["message"])))
+                        raise TileValidationError(
+                            _("{0}".format(error_instance["message"]))
+                        )
             if errors is not None:
                 errors += error
         if not raise_early:
@@ -336,7 +400,11 @@ class Tile(models.TileModel):
             user_id = str(user_id)
             user = User.objects.get(pk=user_id)
             user_is_reviewer = user_is_resource_reviewer(user)
-            if user_is_reviewer is False and self.provisionaledits is not None and user_id in self.provisionaledits:
+            if (
+                user_is_reviewer is False
+                and self.provisionaledits is not None
+                and user_id in self.provisionaledits
+            ):
                 data = self.provisionaledits[user_id]["value"]
 
         return data
@@ -356,7 +424,19 @@ class Tile(models.TileModel):
 
         tile_data = self.get_tile_data(userid)
         for nodeid in tile_data.keys():
-            node = models.Node.objects.get(nodeid=nodeid)
+            try:
+                node = SimpleNamespace(
+                    **next(
+                        (
+                            x
+                            for x in self.serialized_graph["nodes"]
+                            if x["nodeid"] == nodeid
+                        ),
+                        None,
+                    )
+                )
+            except:
+                node = models.Node.objects.get(nodeid=nodeid)
             datatype = self.datatype_factory.get_instance(node.datatype)
             datatype.post_tile_save(self, nodeid, request)
 
@@ -365,7 +445,10 @@ class Tile(models.TileModel):
         index = kwargs.pop("index", True)
         user = kwargs.pop("user", None)
         new_resource_created = kwargs.pop("new_resource_created", False)
+        resource_creation = kwargs.pop("resource_creation", False)
+        note = "resource creation" if resource_creation else None
         context = kwargs.pop("context", None)
+        resource = kwargs.pop("resource", None)
         transaction_id = kwargs.pop("transaction_id", None)
         provisional_edit_log_details = kwargs.pop("provisional_edit_log_details", None)
         creating_new_tile = True
@@ -373,6 +456,8 @@ class Tile(models.TileModel):
         newprovisionalvalue = None
         oldprovisionalvalue = None
 
+        if not self.serialized_graph:
+            self.load_serialized_graph(raise_if_missing=True)
         try:
             if user is None and request is not None:
                 user = request.user
@@ -382,14 +467,20 @@ class Tile(models.TileModel):
 
         with transaction.atomic():
             for nodeid in self.data.keys():
-                node = models.Node.objects.get(nodeid=nodeid)
-                datatype = self.datatype_factory.get_instance(node.datatype)
+                node = next(
+                    item
+                    for item in self.serialized_graph["nodes"]
+                    if item["nodeid"] == nodeid
+                )
+                datatype = self.datatype_factory.get_instance(node["datatype"])
                 datatype.pre_tile_save(self, nodeid)
             self.__preSave(request, context=context)
             self.check_for_missing_nodes()
             self.check_for_constraint_violation()
 
-            creating_new_tile = models.TileModel.objects.filter(pk=self.tileid).exists() is False
+            creating_new_tile = (
+                models.TileModel.objects.filter(pk=self.tileid).exists() is False
+            )
             edit_type = "tile create" if (creating_new_tile is True) else "tile edit"
 
             if creating_new_tile is False:
@@ -407,7 +498,9 @@ class Tile(models.TileModel):
 
                 else:
                     # the user has previously edited this tile
-                    self.apply_provisional_edit(user, self.data, action="update", existing_model=existing_model)
+                    self.apply_provisional_edit(
+                        user, self.data, action="update", existing_model=existing_model
+                    )
                     newprovisionalvalue = self.data
                     self.data = existing_model.data
 
@@ -442,6 +535,7 @@ class Tile(models.TileModel):
                     provisional_edit_log_details=provisional_edit_log_details,
                     transaction_id=transaction_id,
                     new_resource_created=new_resource_created,
+                    note=note,
                 )
             else:
                 self.save_edit(
@@ -455,18 +549,30 @@ class Tile(models.TileModel):
                     transaction_id=transaction_id,
                 )
 
-            if index:
-                self.index()
-
             for tile in self.tiles:
                 tile.resourceinstance = self.resourceinstance
                 tile.parenttile = self
-                tile.save(*args, request=request, index=index, **kwargs)
+                tile.save(
+                    *args,
+                    request=request,
+                    resource_creation=resource_creation,
+                    index=False,
+                    **kwargs,
+                )
+
+            if resource is None:
+                resource = Resource.objects.get(pk=self.resourceinstance_id)
+            resource.save_descriptors(context={"tile": self})
+
+            if index:
+                self.index(resource=resource)
 
     def populate_missing_nodes(self):
         first_node = next(iter(self.data.items()), None)
         if first_node is not None:
-            result = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=self.nodegroup_id)
+            result = Tile.get_blank_tile_from_nodegroup_id(
+                nodegroup_id=self.nodegroup_id
+            )
             result.data.update(self.data)
             self.data = result.data
 
@@ -504,11 +610,28 @@ class Tile(models.TileModel):
             try:
                 super(Tile, self).delete(*args, **kwargs)
                 for nodeid in self.data.keys():
-                    node = models.Node.objects.get(nodeid=nodeid)
+                    try:
+                        node = SimpleNamespace(
+                            **next(
+                                (
+                                    x
+                                    for x in self.serialized_graph["nodes"]
+                                    if x["nodeid"] == nodeid
+                                ),
+                                None,
+                            )
+                        )
+                    except TypeError:  # will catch if serialized_graph is None
+                        node = models.Node.objects.get(nodeid=nodeid)
+
                     datatype = self.datatype_factory.get_instance(node.datatype)
                     datatype.post_tile_delete(self, nodeid, index=index)
+
+                resource = Resource.objects.get(pk=self.resourceinstance_id)
+                resource.save_descriptors()
+
                 if index:
-                    self.index()
+                    self.index(resource=resource)
             except IntegrityError as e:
                 logger.error(e)
 
@@ -516,12 +639,16 @@ class Tile(models.TileModel):
             self.apply_provisional_edit(user, data={}, action="delete")
             super(Tile, self).save(*args, **kwargs)
 
-    def index(self):
+    def index(self, resource=None):
         """
         Indexes all the nessesary documents related to resources to support the map, search, and reports
 
         """
-        Resource.objects.get(pk=self.resourceinstance_id).index()
+
+        if not resource:
+            Resource.objects.get(pk=self.resourceinstance_id).index()
+        else:
+            resource.index()
 
     # # flatten out the nested tiles into a single array
     def get_flattened_tiles(self):
@@ -536,8 +663,16 @@ class Tile(models.TileModel):
         return tiles
 
     def after_update_all(self):
-        nodegroup = models.NodeGroup.objects.get(pk=self.nodegroup_id)
-        for node in nodegroup.node_set.all():
+        try:
+            nodes = [
+                SimpleNamespace(node)
+                for node in self.serialized_graph["nodes"]
+                if node["nodegroup_id"] == str(self.nodegroup_id)
+            ]
+        except TypeError:  # handle if serialized_graph is None
+            nodes = self.nodegroup.node_set.all()
+
+        for node in nodes:
             datatype = self.datatype_factory.get_instance(node.datatype)
             datatype.after_update_all(tile=self)
         for tile in self.tiles:
@@ -545,7 +680,7 @@ class Tile(models.TileModel):
 
     def is_blank(self):
         if self.data != {}:
-            if len([item for item in list(self.data.values()) if item is not None]) > 0:
+            if any(self.data.values()):
                 return False
 
         child_tiles_are_blank = True
@@ -561,31 +696,53 @@ class Tile(models.TileModel):
         node = models.Node.objects.filter(pk=nodeid).select_related("nodegroup")[0]
         parentnodegroup_id = node.nodegroup.parentnodegroup_id
         if parentnodegroup_id is not None:
-            parent_tile = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=parentnodegroup_id, resourceid=resourceid, parenttile=None)
+            parent_tile = Tile.get_blank_tile_from_nodegroup_id(
+                nodegroup_id=parentnodegroup_id, resourceid=resourceid, parenttile=None
+            )
             parent_tile.tileid = None
             parent_tile.tiles = []
-            for nodegroup in models.NodeGroup.objects.filter(parentnodegroup_id=parentnodegroup_id):
-                parent_tile.tiles.append(Tile.get_blank_tile_from_nodegroup_id(nodegroup.pk, resourceid=resourceid, parenttile=parent_tile))
+            for nodegroup in models.NodeGroup.objects.filter(
+                parentnodegroup_id=parentnodegroup_id
+            ):
+                parent_tile.tiles.append(
+                    Tile.get_blank_tile_from_nodegroup_id(
+                        nodegroup.pk, resourceid=resourceid, parenttile=parent_tile
+                    )
+                )
             return parent_tile
         else:
-            return Tile.get_blank_tile_from_nodegroup_id(node.nodegroup_id, resourceid=resourceid)
+            return Tile.get_blank_tile_from_nodegroup_id(
+                node.nodegroup_id, resourceid=resourceid
+            )
 
     @staticmethod
-    def get_blank_tile_from_nodegroup_id(nodegroup_id, resourceid=None, parenttile=None):
+    def get_blank_tile_from_nodegroup_id(
+        nodegroup_id, resourceid=None, parenttile=None
+    ):
         tile = Tile()
         tile.nodegroup_id = nodegroup_id
         tile.resourceinstance_id = resourceid
         tile.parenttile = parenttile
         tile.data = {}
+        nodes = models.Node.objects.filter(nodegroup=nodegroup_id).exclude(
+            datatype="semantic"
+        )
 
-        for node in models.Node.objects.filter(nodegroup=nodegroup_id):
-            if node.datatype != "semantic":
-                tile.data[str(node.nodeid)] = None
+        for node in nodes:
+            tile.data[str(node.nodeid)] = None
 
         return tile
 
     @staticmethod
-    def update_node_value(nodeid, value, tileid=None, nodegroupid=None, request=None, resourceinstanceid=None, transaction_id=None):
+    def update_node_value(
+        nodeid,
+        value,
+        tileid=None,
+        nodegroupid=None,
+        request=None,
+        resourceinstanceid=None,
+        transaction_id=None,
+    ):
         """
         Updates the value of a node in a tile. Creates the tile and parent tiles if they do not yet
         exist.
@@ -595,8 +752,15 @@ class Tile(models.TileModel):
             tile = Tile.objects.get(pk=tileid)
             tile.data[nodeid] = value
             tile.save(request=request, transaction_id=transaction_id)
-        elif models.TileModel.objects.filter(Q(resourceinstance_id=resourceinstanceid), Q(nodegroup_id=nodegroupid)).count() == 1:
-            tile = Tile.objects.filter(Q(resourceinstance_id=resourceinstanceid), Q(nodegroup_id=nodegroupid))[0]
+        elif (
+            models.TileModel.objects.filter(
+                Q(resourceinstance_id=resourceinstanceid), Q(nodegroup_id=nodegroupid)
+            ).count()
+            == 1
+        ):
+            tile = Tile.objects.filter(
+                Q(resourceinstance_id=resourceinstanceid), Q(nodegroup_id=nodegroupid)
+            )[0]
             tile.data[nodeid] = value
             tile.save(request=request, transaction_id=transaction_id)
         else:
@@ -610,9 +774,17 @@ class Tile(models.TileModel):
             tile = Tile.get_blank_tile(nodeid, resourceinstanceid)
             if nodeid in tile.data:
                 tile.data[nodeid] = value
-                tile.save(request=request, new_resource_created=new_resource_created, transaction_id=transaction_id)
+                tile.save(
+                    request=request,
+                    new_resource_created=new_resource_created,
+                    transaction_id=transaction_id,
+                )
             else:
-                tile.save(request=request, new_resource_created=new_resource_created, transaction_id=transaction_id)
+                tile.save(
+                    request=request,
+                    new_resource_created=new_resource_created,
+                    transaction_id=transaction_id,
+                )
                 if not nodegroupid:
                     nodegroupid = models.Node.objects.get(pk=nodeid).nodegroup_id
                 if nodegroupid and resourceinstanceid:
@@ -642,7 +814,9 @@ class Tile(models.TileModel):
                 except NotImplementedError:
                     pass
         except TypeError as e:
-            logger.warning(_("No associated functions or other TypeError raised by a function"))
+            logger.warning(
+                _("No associated functions or other TypeError raised by a function")
+            )
             logger.warning(e)
 
     def __preDelete(self, request):
@@ -653,7 +827,9 @@ class Tile(models.TileModel):
                 except NotImplementedError:
                     pass
         except TypeError as e:
-            logger.warning(_("No associated functions or other TypeError raised by a function"))
+            logger.warning(
+                _("No associated functions or other TypeError raised by a function")
+            )
             logger.warning(e)
 
     def __postSave(self, request=None, context=None):
@@ -670,7 +846,9 @@ class Tile(models.TileModel):
                 except NotImplementedError:
                     pass
         except TypeError as e:
-            logger.warning(_("No associated functions or other TypeError raised by a function"))
+            logger.warning(
+                _("No associated functions or other TypeError raised by a function")
+            )
             logger.warning(e)
 
     def _getFunctionClassInstances(self):
@@ -678,18 +856,23 @@ class Tile(models.TileModel):
         resource = models.ResourceInstance.objects.get(pk=self.resourceinstance_id)
         functionXgraphs = models.FunctionXGraph.objects.filter(
             Q(graph_id=resource.graph_id),
-            Q(config__contains={"triggering_nodegroups": [str(self.nodegroup_id)]}) | Q(config__triggering_nodegroups__exact=[]),
+            Q(config__contains={"triggering_nodegroups": [str(self.nodegroup_id)]})
+            | Q(config__triggering_nodegroups__exact=[]),
             ~Q(function__functiontype="primarydescriptors"),
         )
         for functionXgraph in functionXgraphs:
-            func = functionXgraph.function.get_class_module()(functionXgraph.config, self.nodegroup_id)
+            func = functionXgraph.function.get_class_module()(
+                functionXgraph.config, self.nodegroup_id
+            )
             ret.append(func)
         return ret
 
     def filter_by_perm(self, user, perm):
         if user:
             if self.nodegroup_id is not None and user.has_perm(perm, self.nodegroup):
-                self.tiles = [tile for tile in self.tiles if tile.filter_by_perm(user, perm)]
+                self.tiles = [
+                    tile for tile in self.tiles if tile.filter_by_perm(user, perm)
+                ]
             else:
                 return None
         return self
@@ -719,4 +902,4 @@ class TileValidationError(Exception):
 class TileCardinalityError(TileValidationError):
     def __init__(self, message, code=None):
         super(TileCardinalityError, self).__init__(message, code)
-        self.title = _("Tile Cardinaltiy Error")
+        self.title = _("Tile Cardinality Error")
