@@ -4,9 +4,8 @@ details = {
     "icon": "fa fa-code-fork",
     "modulename": "",
     "classname": "",
-    "type": "filter",
+    "type": "related-resources-filter-type",
     "componentpath": "views/components/search/related-resources-filter",
     "componentname": "related-resources-filter",
-    "sortorder": "4",
-    "enabled": True,
+    "config": {},
 }
