@@ -1,3 +1,4 @@
+import logging
 import math
 from arches.app.utils.date_utils import ExtendedDateFormat
 from arches.app.utils.permission_backend import get_nodegroups_by_perm
@@ -23,6 +24,9 @@ from arches.app.search.search_engine_factory import SearchEngineFactory
 from arches.app.search.mappings import RESOURCES_INDEX
 from arches.app.models.system_settings import settings
 from django.core.cache import cache
+from django.utils.translation import get_language, gettext as _
+
+logger = logging.getLogger(__name__)
 
 
 class TimeWheel(object):
@@ -37,28 +41,61 @@ class TimeWheel(object):
 
         if (
             results is not None
-            and results["aggregations"]["min_max_agg"]["min_dates.date"]["value"] is not None
-            and results["aggregations"]["min_max_agg"]["max_dates.date"]["value"] is not None
+            and results["aggregations"]["min_max_agg"]["min_dates.date"]["value"]
+            is not None
+            and results["aggregations"]["min_max_agg"]["max_dates.date"]["value"]
+            is not None
         ):
-            min_date = int(results["aggregations"]["min_max_agg"]["min_dates.date"]["value"]) / 10000
-            max_date = int(results["aggregations"]["min_max_agg"]["max_dates.date"]["value"]) / 10000
+            min_date = (
+                int(results["aggregations"]["min_max_agg"]["min_dates.date"]["value"])
+                / 10000
+            )
+            max_date = (
+                int(results["aggregations"]["min_max_agg"]["max_dates.date"]["value"])
+                / 10000
+            )
             # round min and max date to the nearest 1000 years
-            min_date = math.ceil(math.fabs(min_date) / 1000) * -1000 if min_date < 0 else math.floor(min_date / 1000) * 1000
-            max_date = math.floor(math.fabs(max_date) / 1000) * -1000 if max_date < 0 else math.ceil(max_date / 1000) * 1000
+            min_date = (
+                math.ceil(math.fabs(min_date) / 1000) * -1000
+                if min_date < 0
+                else math.floor(min_date / 1000) * 1000
+            )
+            max_date = (
+                math.floor(math.fabs(max_date) / 1000) * -1000
+                if max_date < 0
+                else math.ceil(max_date / 1000) * 1000
+            )
             query = Query(se, limit=0)
             range_lookup = {}
 
             def gen_range_agg(gte=None, lte=None, permitted_nodegroups=None):
                 date_query = Bool()
-                date_query.filter(Range(field="dates.date", gte=gte, lte=lte, relation="intersects"))
+                date_query.filter(
+                    Range(field="dates.date", gte=gte, lte=lte, relation="intersects")
+                )
                 if permitted_nodegroups is not None:
-                    date_query.filter(Terms(field="dates.nodegroup_id", terms=permitted_nodegroups))
+                    date_query.filter(
+                        Terms(field="dates.nodegroup_id", terms=permitted_nodegroups)
+                    )
                 date_ranges_query = Bool()
-                date_ranges_query.filter(Range(field="date_ranges.date_range", gte=gte, lte=lte, relation="intersects"))
+                date_ranges_query.filter(
+                    Range(
+                        field="date_ranges.date_range",
+                        gte=gte,
+                        lte=lte,
+                        relation="intersects",
+                    )
+                )
                 if permitted_nodegroups is not None:
-                    date_ranges_query.filter(Terms(field="date_ranges.nodegroup_id", terms=permitted_nodegroups))
+                    date_ranges_query.filter(
+                        Terms(
+                            field="date_ranges.nodegroup_id", terms=permitted_nodegroups
+                        )
+                    )
                 wrapper_query = Bool()
-                wrapper_query.should(Nested(path="date_ranges", query=date_ranges_query))
+                wrapper_query.should(
+                    Nested(path="date_ranges", query=date_ranges_query)
+                )
                 wrapper_query.should(Nested(path="dates", query=date_query))
                 return wrapper_query
 
@@ -66,7 +103,11 @@ class TimeWheel(object):
                 "name": "Millennium",
                 "interval": 1000,
                 "root": True,
-                "child": {"name": "Century", "interval": 100, "child": {"name": "Decade", "interval": 10}},
+                "child": {
+                    "name": "Century",
+                    "interval": 100,
+                    "child": {"name": "Decade", "interval": 10},
+                },
             }
 
             if abs(int(min_date) - int(max_date)) > 1000:
@@ -74,7 +115,11 @@ class TimeWheel(object):
                     "name": "Millennium",
                     "interval": 1000,
                     "root": True,
-                    "child": {"name": "Half-millennium", "interval": 500, "child": {"name": "Century", "interval": 100}},
+                    "child": {
+                        "name": "Half-millennium",
+                        "interval": 500,
+                        "child": {"name": "Century", "interval": 100},
+                    },
                 }
 
             if settings.TIMEWHEEL_DATE_TIERS is not None:
@@ -90,10 +135,19 @@ class TimeWheel(object):
                     min_period = period
                     max_period = period + interval
                     if "range" in date_tier:
-                        within_range = min_period >= date_tier["range"]["min"] and max_period <= date_tier["range"]["max"]
+                        within_range = (
+                            min_period >= date_tier["range"]["min"]
+                            and max_period <= date_tier["range"]["max"]
+                        )
                     if within_range is True:
-                        period_name = "{0} ({1} - {2})".format(name, min_period, max_period)
-                        nodegroups = self.get_permitted_nodegroups(user) if "root" in date_tier else None
+                        period_name = "{0} ({1} - {2})".format(
+                            name, min_period, max_period
+                        )
+                        nodegroups = (
+                            self.get_permitted_nodegroups(user)
+                            if "root" in date_tier
+                            else None
+                        )
                         period_boolquery = gen_range_agg(
                             gte=ExtendedDateFormat(min_period).lower,
                             lte=ExtendedDateFormat(max_period).lower,
@@ -105,7 +159,9 @@ class TimeWheel(object):
                             previous_period_agg.add_aggregation(period_agg)
                         range_lookup[period_name] = [min_period, max_period]
                         if "child" in date_tier:
-                            add_date_tier(date_tier["child"], min_period, max_period, period_agg)
+                            add_date_tier(
+                                date_tier["child"], min_period, max_period, period_agg
+                            )
                         if "root" in date_tier:
                             query.add_aggregation(period_agg)
 
@@ -120,9 +176,18 @@ class TimeWheel(object):
             for child in root.children:
                 root.size = root.size + child.size
 
+            key = "time_wheel_config_{0}".format(user.username)
             if user.username in settings.CACHE_BY_USER:
-                key = "time_wheel_config_{0}".format(user.username)
                 cache.set(key, root, settings.CACHE_BY_USER[user.username])
+            else:
+                try:
+                    cache.set(key, root, settings.CACHE_BY_USER["default"])
+                except KeyError:
+                    logger.warning(
+                        _(
+                            "CACHE_BY_USER setting does not have a 'default'. Adding a default can improve search page performance."
+                        )
+                    )
 
             return root
 
@@ -145,7 +210,9 @@ class TimeWheel(object):
                 if item.size > 0:
                     d3ItemInstance.children.append(item)
 
-        d3ItemInstance.children = sorted(d3ItemInstance.children, key=lambda item: item.start)
+        d3ItemInstance.children = sorted(
+            d3ItemInstance.children, key=lambda item: item.start
+        )
 
         return d3ItemInstance
 
@@ -161,7 +228,7 @@ class TimeWheel(object):
         return results
 
     def get_permitted_nodegroups(self, user):
-        return [str(nodegroup.pk) for nodegroup in get_nodegroups_by_perm(user, "models.read_nodegroup")]
+        return get_nodegroups_by_perm(user, "models.read_nodegroup")
 
 
 class d3Item(object):
