@@ -21,7 +21,7 @@ from arches.app.models.fields.i18n import I18n_JSON, I18n_String
 
 
 class UnableToSerializeError(Exception):
-    """ Error for not implemented classes """
+    """Error for not implemented classes"""
 
     def __init__(self, value):
         self.value = value
@@ -32,7 +32,7 @@ class UnableToSerializeError(Exception):
 
 
 class UnableToSerializeMethodTypesError(Exception):
-    """ Error for not implemented classes """
+    """Error for not implemented classes"""
 
     def __init__(self, value):
         self.value = value
@@ -43,21 +43,39 @@ class UnableToSerializeMethodTypesError(Exception):
 
 
 class JSONSerializer(object):
-    def serializeToPython(self, obj, **options):
-        self.options = options.copy()
+    """
+    You can pass this class as a JSON Encoder to the json.dumps "cls" property
+    eg: json.dumps(obj, cls=JSONSerializer)
+    """
 
-        self.stream = options.pop("stream", StringIO())
-        self.selected_fields = options.pop("fields", None)
-        self.exclude = options.pop("exclude", None)
-        self.use_natural_keys = options.pop("use_natural_keys", False)
-        self.geom_format = options.pop("geom_format", "wkt")
-        self.force_recalculation = options.pop("force_recalculation", False)
+    def __init__(self, **options):
+        self._options = options
+        self.utf_encode = False
+
+    def encode(self, obj):
+        if isinstance(obj, (I18n_JSON, I18n_String)):
+            # Ensure raw JSON strings are re-encoded ("'" -> "\"")
+            # and non-active language values are preserved.
+            return DjangoJSONEncoder().encode(obj.raw_value)
+        return self.serialize(obj, **self._options)
+
+    def serializeToPython(self, obj, **options):
+        # allow users to override any kwargs passed into the __init__ method
+        self.options = self._options.copy()
+        self.options.update(options)
+
+        self.stream = self.options.get("stream", StringIO())
+        self.selected_fields = self.options.get("fields", None)
+        self.exclude = self.options.get("exclude", None)
+        self.use_natural_keys = self.options.get("use_natural_keys", False)
+        self.geom_format = self.options.get("geom_format", "wkt")
+        self.force_recalculation = self.options.get("force_recalculation", False)
 
         return self.handle_object(obj, **self.options)
 
     def serialize(self, obj, **options):
         obj = self.serializeToPython(obj, **options)
-        # prevent raw strings from begin re-encoded
+        # prevent raw strings from being re-encoded
         # this is especially important when doing bulk operations in elasticsearch
         if isinstance(obj, str):
             return obj
@@ -66,7 +84,11 @@ class JSONSerializer(object):
         options.pop("fields", None)
         options.pop("exclude", None)
         options.pop("force_recalculation", False)
-        return json.dumps(obj, cls=DjangoJSONEncoder, sort_keys=sort_keys, **options.copy())
+        result = json.dumps(
+            obj, cls=DjangoJSONEncoder, sort_keys=sort_keys, **options.copy()
+        )
+
+        return result.encode("utf-8") if self.utf_encode else result
 
     def handle_object(self, object, **kwargs):
         """Called to handle everything, looks for the correct handling"""
@@ -79,11 +101,19 @@ class JSONSerializer(object):
         # print inspect.isroutine(object)
         # print inspect.isabstract(object)
         # print type(object) == 'staticmethod'
-        if inspect.isroutine(object) or inspect.isbuiltin(object) or inspect.isclass(object):
+        if (
+            inspect.isroutine(object)
+            or inspect.isbuiltin(object)
+            or inspect.isclass(object)
+        ):
             raise UnableToSerializeMethodTypesError(type(object))
         elif isinstance(object, dict):
             return self.handle_dictionary(object)
-        elif isinstance(object, list) or isinstance(object, tuple) or isinstance(object, set):
+        elif (
+            isinstance(object, list)
+            or isinstance(object, tuple)
+            or isinstance(object, set)
+        ):
             return self.handle_list(object)
         elif isinstance(object, Model):
             if hasattr(object, "serialize"):
@@ -93,7 +123,9 @@ class JSONSerializer(object):
                 if self.force_recalculation:
                     signature = inspect.signature(serialize_function)
 
-                    if "force_recalculation" in [parameter.name for parameter in signature.parameters.values()]:
+                    if "force_recalculation" in [
+                        parameter.name for parameter in signature.parameters.values()
+                    ]:
                         kwargs["force_recalculation"] = True
                         # return self.handle_object(serialize_function(**kwargs), **kwargs)
 
@@ -150,7 +182,7 @@ class JSONSerializer(object):
             try:
                 # print key + ': ' + str(type(value))
                 obj[str(key)] = self.handle_object(value)
-            except (UnableToSerializeMethodTypesError):
+            except UnableToSerializeMethodTypesError:
                 pass
 
         return obj
@@ -184,16 +216,18 @@ class JSONSerializer(object):
         fields = kwargs.get("fields", None)
         exclude = kwargs.get("exclude", None)
         # print '='*40
-        properties = [k for k, v in instance.__class__.__dict__.items() if type(v) is property]
+        properties = [
+            k for k, v in instance.__class__.__dict__.items() if type(v) is property
+        ]
         for property_name in properties:
             if fields and property_name not in fields:
                 continue
             if exclude and property_name in exclude:
                 continue
-            data[property_name] = self.handle_object(getattr(instance, property_name), **kwargs)
+            data[property_name] = self.handle_object(
+                getattr(instance, property_name), **kwargs
+            )
         for f in chain(opts.concrete_fields, opts.private_fields, opts.many_to_many):
-            if not getattr(f, "editable", False):
-                continue
             if fields and f.name not in fields:
                 continue
             if exclude and f.name in exclude:
@@ -215,7 +249,9 @@ class JSONSerializer(object):
                     qs = f.value_from_object(instance)
                     data[f.name] = [item.pk for item in qs]
             else:
-                data[f.name] = self.handle_object(f.value_from_object(instance), **kwargs)
+                data[f.name] = self.handle_object(
+                    f.value_from_object(instance), **kwargs
+                )
         return data
 
 
@@ -255,7 +291,7 @@ class JSONDeserializer(object):
         return ret
 
     def handle_object(self, object, fields=None, exclude=None):
-        """ Called to handle everything, looks for the correct handling """
+        """Called to handle everything, looks for the correct handling"""
         if isinstance(object, dict):
             if "pk" in object and "model" in object and "fields" in object:
                 # assume that this is a serialized django model
