@@ -66,7 +66,7 @@ class GraphImportException(Exception):
     pass
 
 
-def import_graph(graphs, overwrite_graphs=True):
+def import_graph(graphs, overwrite_graphs=True, user=None):
     reporter = GraphImportReporter(graphs)
 
     def check_default_configs(default_configs, configs):
@@ -87,12 +87,19 @@ def import_graph(graphs, overwrite_graphs=True):
         errors = []
         for resource in graphs:
             if len(list(OntologyClass.objects.all())) > 0:
-                if resource["ontology_id"] is not None and resource["ontology_id"] not in [
-                    str(f["ontologyid"]) for f in Ontology.objects.all().values("ontologyid")
+                if resource["ontology_id"] is not None and resource[
+                    "ontology_id"
+                ] not in [
+                    str(f["ontologyid"])
+                    for f in Ontology.objects.all().values("ontologyid")
                 ]:
-                    errors.append("The ontologyid of the graph you're trying to load does not exist in Arches.")
+                    errors.append(
+                        "The ontologyid of the graph you're trying to load does not exist in Arches."
+                    )
             else:
-                errors.append("No ontologies have been loaded. Any GraphModel that depends on an ontology cannot be loaded.")
+                logger.warning(
+                    "No ontologies have been loaded. Any GraphModel that depends on an ontology cannot be loaded."
+                )
 
             reporter.name = resource["name"]
             reporter.resource_model = resource["isresource"]
@@ -104,56 +111,90 @@ def import_graph(graphs, overwrite_graphs=True):
 
             try:
                 graph = Graph(resource)
-                ontology_classes = [str(f["source"]) for f in OntologyClass.objects.all().values("source")]
+                ontology_classes = [
+                    str(f["source"])
+                    for f in OntologyClass.objects.all().values("source")
+                ]
 
                 for node in list(graph.nodes.values()):
                     if resource["ontology_id"] is not None:
                         if node.ontologyclass not in ontology_classes:
-                            errors.append("The ontology class of this node does not exist in the indicated ontology scheme.")
+                            errors.append(
+                                "The ontology class of this node does not exist in the indicated ontology scheme."
+                            )
                     node_config = node.config
-                    default_config = DDataType.objects.get(datatype=node.datatype).defaultconfig
+                    default_config = DDataType.objects.get(
+                        datatype=node.datatype
+                    ).defaultconfig
                     node.config = check_default_configs(default_config, node_config)
 
                 if not hasattr(graph, "cards"):
                     errors.append("{0} graph has no attribute cards".format(graph.name))
                 else:
-                    if len(Graph.objects.filter(pk=graph.graphid)) == 0 or overwrite_graphs is True:
+                    if (
+                        len(Graph.objects.filter(pk=graph.graphid)) == 0
+                        or overwrite_graphs is True
+                    ):
                         if hasattr(graph, "reports"):
                             for report in graph.reports:
                                 if report["active"]:
                                     report_config = report["config"]
-                                    default_config = ReportTemplate.objects.get(templateid=report["template_id"]).defaultconfig
-                                    graph.config = check_default_configs(default_config, report_config)
+                                    default_config = ReportTemplate.objects.get(
+                                        templateid=report["template_id"]
+                                    ).defaultconfig
+                                    graph.config = check_default_configs(
+                                        default_config, report_config
+                                    )
                                     graph.template_id = report["template_id"]
                         graph.save()
                         reporter.update_graphs_saved()
                     else:
-                        overwrite_input = input("Overwrite {0} (Y/N) ? ".format(graph.name))
+                        overwrite_input = input(
+                            "Overwrite {0} (Y/N) ? ".format(graph.name)
+                        )
                         if overwrite_input.lower() in ("t", "true", "y", "yes"):
                             graph.save()
                         else:
-                            raise GraphImportException("{0} - already exists. Skipping import.".format(graph.name))
+                            raise GraphImportException(
+                                "{0} - already exists. Skipping import.".format(
+                                    graph.name
+                                )
+                            )
 
                 if not hasattr(graph, "cards_x_nodes_x_widgets"):
-                    errors.append("{0} graph has no attribute cards_x_nodes_x_widgets".format(graph.name))
+                    errors.append(
+                        "{0} graph has no attribute cards_x_nodes_x_widgets".format(
+                            graph.name
+                        )
+                    )
                 else:
                     for card_x_node_x_widget in graph.cards_x_nodes_x_widgets:
                         card_x_node_x_widget_config = card_x_node_x_widget["config"]
-                        default_config = Widget.objects.get(widgetid=card_x_node_x_widget["widget_id"]).defaultconfig
-                        card_x_node_x_widget["config"] = check_default_configs(default_config, card_x_node_x_widget_config)
-                        cardxnodexwidget = CardXNodeXWidget.objects.update_or_create(**card_x_node_x_widget)
+                        default_config = Widget.objects.get(
+                            widgetid=card_x_node_x_widget["widget_id"]
+                        ).defaultconfig
+                        card_x_node_x_widget["config"] = check_default_configs(
+                            default_config, card_x_node_x_widget_config
+                        )
+                        cardxnodexwidget = CardXNodeXWidget.objects.update_or_create(
+                            **card_x_node_x_widget
+                        )
 
                 with transaction.atomic():
                     # saves graph publication with serialized graph
-                    graph = Graph.objects.get(pk=graph.graphid)  # retrieve graph using the ORM to ensure strings are I18n_Strings
+                    graph = Graph.objects.get(
+                        pk=graph.graphid
+                    )  # retrieve graph using the ORM to ensure strings are I18n_Strings
                     if publication_data:
                         GraphXPublishedGraph.objects.update_or_create(
                             publicationid=publication_data["publicationid"],
                             defaults={
                                 "notes": publication_data.get("notes"),
                                 "graph_id": publication_data.get("graph_id"),
-                                "user_id": publication_data.get("user_id"),
-                                "published_time": publication_data.get("published_time"),
+                                "user_id": user.id if user else None,
+                                "published_time": publication_data.get(
+                                    "published_time"
+                                ),
                             },
                         )
 
@@ -171,7 +212,9 @@ def import_graph(graphs, overwrite_graphs=True):
                                 language=language,
                                 defaults={
                                     "serialized_graph": JSONDeserializer().deserialize(
-                                        JSONSerializer().serialize(graph, force_recalculation=True)
+                                        JSONSerializer().serialize(
+                                            graph, force_recalculation=True
+                                        )
                                     )
                                 },
                             )
@@ -188,11 +231,19 @@ def import_graph(graphs, overwrite_graphs=True):
             try:
                 if not hasattr(graph, "resource_2_resource_constraints"):
                     errors.append(
-                        "{0} graph has no attribute resource_2_resource_constraints".format(graph.resource_2_resource_constraints)
+                        "{0} graph has no attribute resource_2_resource_constraints".format(
+                            graph.resource_2_resource_constraints
+                        )
                     )
                 else:
-                    for resource_2_resource_constraint in graph.resource_2_resource_constraints:
-                        resource2resourceconstraint = Resource2ResourceConstraint.objects.update_or_create(**resource_2_resource_constraint)
+                    for (
+                        resource_2_resource_constraint
+                    ) in graph.resource_2_resource_constraints:
+                        resource2resourceconstraint = (
+                            Resource2ResourceConstraint.objects.update_or_create(
+                                **resource_2_resource_constraint
+                            )
+                        )
             except:
                 pass
 
@@ -203,4 +254,6 @@ def import_mapping_file(mapping_file):
     resource_model_id = mapping_file["resource_model_id"]
     mapping = mapping_file
 
-    GraphXMapping.objects.update_or_create(graph_id=uuid.UUID(str(resource_model_id)), mapping=mapping)
+    GraphXMapping.objects.update_or_create(
+        graph_id=uuid.UUID(str(resource_model_id)), mapping=mapping
+    )
