@@ -23,13 +23,14 @@ import uuid
 import logging
 from django.db import transaction
 from django.shortcuts import redirect, render
-from django.db.models import Q
-from django.utils.translation import ugettext as _
+from django.db.models import Exists, F, Func, OuterRef, Q
+from django.utils.translation import gettext as _
 from django.utils.decorators import method_decorator
 from django.http import HttpResponseNotFound, HttpResponse
 from django.views.generic import View, TemplateView
 from django.contrib.auth.models import User, Group, Permission
 from django.contrib.contenttypes.models import ContentType
+from django.core.exceptions import PermissionDenied, ValidationError
 from arches.app.utils.decorators import group_required
 from arches.app.utils.betterJSONSerializer import JSONSerializer, JSONDeserializer
 from arches.app.utils.response import JSONResponse, JSONErrorResponse
@@ -40,11 +41,20 @@ from arches.app.models.concept import Concept
 from arches.app.models.fields.i18n import I18n_String
 from arches.app.models.system_settings import settings
 from arches.app.models.resource import PublishedModelError, UnpublishedModelError
-from arches.app.utils.data_management.resource_graphs.exporter import get_graphs_for_export, create_mapping_configuration_file
+from arches.app.utils.data_management.resource_graphs.exporter import (
+    get_graphs_for_export,
+    create_mapping_configuration_file,
+)
 from arches.app.utils.data_management.resource_graphs import importer as GraphImporter
 from arches.app.utils.system_metadata import system_metadata
 from arches.app.views.base import BaseManagerView
-from guardian.shortcuts import assign_perm, get_perms, remove_perm, get_group_perms, get_user_perms
+from arches.app.utils.permission_backend import (
+    assign_perm,
+    get_perms,
+    remove_perm,
+    get_group_perms,
+    get_user_perms,
+)
 from io import BytesIO
 from elasticsearch.exceptions import RequestError
 from django.core.cache import cache
@@ -74,18 +84,32 @@ class GraphSettingsView(GraphBaseView):
 
         node = models.Node.objects.get(graph_id=graphid, istopnode=True)
         relatable_resources = node.get_relatable_resources()
-        resource_graphs = models.GraphModel.objects.filter(Q(isresource=True)).exclude(graphid=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID)
+        resource_graphs = models.GraphModel.objects.filter(Q(isresource=True)).exclude(
+            graphid=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID
+        )
 
-        node_models = models.Node.objects.filter(graph__pk__in=[resource_graph.pk for resource_graph in resource_graphs])
+        node_models = models.Node.objects.filter(
+            graph__pk__in=[resource_graph.pk for resource_graph in resource_graphs]
+        )
 
         for res in resource_graphs:
-            node_model = node_models.get(graph=res, istopnode=True)
-            if node_model:
-                resource_data.append({"id": node_model.nodeid, "graph": res, "is_relatable": (node_model in relatable_resources)})
+            try:
+                node_model = node_models.get(graph=res, istopnode=True)
+                resource_data.append(
+                    {
+                        "id": node_model.nodeid,
+                        "graph": res,
+                        "is_relatable": (node_model in relatable_resources),
+                    }
+                )
+            except models.Node.DoesNotExist:
+                pass
 
         return JSONResponse(
             {
-                "icons": JSONSerializer().serializeToPython(models.Icon.objects.order_by("name")),
+                "icons": JSONSerializer().serializeToPython(
+                    models.Icon.objects.order_by("name")
+                ),
                 "node_count": models.Node.objects.filter(graph=self.graph).count(),
                 "resources": JSONSerializer().serializeToPython(resource_data),
             }
@@ -118,7 +142,11 @@ class GraphSettingsView(GraphBaseView):
             node.datatype = data["graph"]["root"]["datatype"]
         except KeyError as e:
             print(e, "Cannot find root node datatype")
-        node.ontologyclass = data.get("ontology_class") if data.get("graph").get("ontology_id") is not None else None
+        node.ontologyclass = (
+            data.get("ontology_class")
+            if data.get("graph").get("ontology_id") is not None
+            else None
+        )
         node.name = graph.name
         graph.root.name = node.name
         if node.ontologyclass:
@@ -127,13 +155,33 @@ class GraphSettingsView(GraphBaseView):
         if graph.isresource is False and "root" in data["graph"]:
             node.config = data["graph"]["root"]["config"]
 
+        nodegroup_ids_to_serialized_nodegroups = {}
+        for serialized_nodegroup in data["graph"]["nodegroups"]:
+            nodegroup_ids_to_serialized_nodegroups[
+                serialized_nodegroup["nodegroupid"]
+            ] = serialized_nodegroup
+
         try:
             with transaction.atomic():
                 graph.save()
                 node.save()
 
+                for nodegroup in models.NodeGroup.objects.filter(
+                    nodegroupid__in=nodegroup_ids_to_serialized_nodegroups.keys()
+                ):
+                    nodegroup.cardinality = nodegroup_ids_to_serialized_nodegroups[
+                        str(nodegroup.nodegroupid)
+                    ]["cardinality"]
+                    nodegroup.save()
+
             return JSONResponse(
-                {"success": True, "graph": graph, "relatable_resource_ids": [res.nodeid for res in node.get_relatable_resources()]}
+                {
+                    "success": True,
+                    "graph": graph,
+                    "relatable_resource_ids": [
+                        res.nodeid for res in node.get_relatable_resources()
+                    ],
+                }
             )
 
         except GraphValidationError as e:
@@ -145,13 +193,23 @@ class GraphManagerView(GraphBaseView):
     def get(self, request, graphid):
         if graphid is None or graphid == "":
             root_nodes = models.Node.objects.filter(istopnode=True)
-            context = self.get_context_data(main_script="views/graph", root_nodes=JSONSerializer().serialize(root_nodes))
-            context["graph_models"] = models.GraphModel.objects.all().exclude(graphid=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID)
-            context["graphs"] = JSONSerializer().serialize(context["graph_models"], exclude=["functions"])
+            context = self.get_context_data(
+                main_script="views/graph",
+                root_nodes=JSONSerializer().serialize(root_nodes),
+            )
+            context["graph_models"] = models.GraphModel.objects.all().exclude(
+                graphid=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID
+            )
+            context["graphs"] = JSONSerializer().serialize(
+                context["graph_models"], exclude=["functions"]
+            )
             context["nav"]["title"] = _("Arches Designer")
             context["nav"]["icon"] = "fa-bookmark"
 
-            context["nav"]["help"] = {"title": _("Using the Arches Designer"), "template": "arches-designer-help"}
+            context["nav"]["help"] = {
+                "title": _("Using the Arches Designer"),
+                "templates": ["arches-designer-help"],
+            }
             return render(request, "views/graph.htm", context)
 
 
@@ -176,43 +234,67 @@ class GraphDesignerView(GraphBaseView):
         return ontology_namespaces
 
     def get(self, request, graphid):
+        if graphid == settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID:
+            if not group_required("System Administrator", raise_exception=True):
+                raise PermissionDenied
+
         self.graph = Graph.objects.get(graphid=graphid)
-        serialized_graph = self.graph.serialize(force_recalculation=True)  # calling `serialize` directly returns a dict
+        serialized_graph = self.graph.serialize(
+            force_recalculation=True
+        )  # calling `serialize` directly returns a dict
 
         datatypes = models.DDataType.objects.all()
-        primary_descriptor_functions = models.FunctionXGraph.objects.filter(graph=self.graph).filter(
-            function__functiontype="primarydescriptors"
-        )
+        primary_descriptor_functions = models.FunctionXGraph.objects.filter(
+            graph=self.graph
+        ).filter(function__functiontype="primarydescriptors")
         primary_descriptor_function = JSONSerializer().serialize(
-            primary_descriptor_functions[0] if len(primary_descriptor_functions) > 0 else None
+            primary_descriptor_functions[0]
+            if len(primary_descriptor_functions) > 0
+            else None
         )
         widgets = models.Widget.objects.all()
         card_components = models.CardComponent.objects.all()
-        graph_models = models.GraphModel.objects.all().exclude(graphid=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID)
+        graph_models = models.GraphModel.objects.all().exclude(
+            graphid=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID
+        )
 
         branch_graphs = Graph.objects.exclude(pk=graphid).exclude(isresource=True)
         if self.graph.ontology is not None:
             branch_graphs = branch_graphs.filter(ontology=self.graph.ontology)
 
-        restricted_nodegroups = []
         if not settings.OVERRIDE_RESOURCE_MODEL_LOCK:
-            restricted_nodegroups = (
-                models.TileModel.objects.filter(
-                    nodegroup__pk__in=[nodegroup_dict["nodegroupid"] for nodegroup_dict in serialized_graph["nodegroups"]]
-                )
-                .values_list("nodegroup_id", flat=True)
-                .distinct()
+            restricted_nodegroups = models.NodeGroup.objects.filter(
+                Exists(models.TileModel.objects.filter(nodegroup=OuterRef("pk"))),
+                pk__in=[
+                    nodegroup_dict["nodegroupid"]
+                    for nodegroup_dict in serialized_graph["nodegroups"]
+                ],
             )
+        else:
+            restricted_nodegroups = models.NodeGroup.objects.none()
 
         context = self.get_context_data(
             main_script="views/graph-designer",
-            datatypes_json=JSONSerializer().serialize(datatypes, exclude=["modulename", "isgeometric"]),
+            datatypes_json=JSONSerializer().serialize(
+                datatypes, exclude=["modulename", "isgeometric"]
+            ),
             datatypes=datatypes,
             ontology_namespaces=self.get_ontology_namespaces(),
             branches=JSONSerializer().serialize(
-                branch_graphs, exclude=["cards", "domain_connections", "functions", "cards", "deploymentfile", "deploymentdate"]
+                branch_graphs,
+                exclude=[
+                    "cards",
+                    "domain_connections",
+                    "functions",
+                    "cards",
+                    "deploymentfile",
+                    "deploymentdate",
+                ],
             ),
-            branch_list={"title": _("Branch Library"), "search_placeholder": _("Find a graph branch")},
+            branch_list={
+                "title": _("Branch Library"),
+                "search_placeholder": _("Find a graph branch"),
+            },
             widgets=widgets,
             widgets_json=JSONSerializer().serialize(widgets),
             card_components=card_components,
@@ -226,12 +308,23 @@ class GraphDesignerView(GraphBaseView):
             primary_descriptor_function=primary_descriptor_function,
             geocoding_providers=models.Geocoder.objects.all(),
             report_templates=models.ReportTemplate.objects.all(),
-            restricted_nodegroups=[str(nodegroup) for nodegroup in restricted_nodegroups],
-            ontologies=JSONSerializer().serialize(models.Ontology.objects.filter(parentontology=None), exclude=["version", "path"]),
-            ontology_classes=JSONSerializer().serialize(models.OntologyClass.objects.values("source", "ontology_id")),
+            restricted_nodegroups=[
+                str(nodegroup.pk) for nodegroup in restricted_nodegroups
+            ],
+            ontologies=JSONSerializer().serialize(
+                models.Ontology.objects.filter(parentontology=None),
+                exclude=["version", "path"],
+            ),
+            ontology_classes=JSONSerializer().serialize(
+                models.OntologyClass.objects.values("source", "ontology_id")
+            ),
             graph_models=graph_models,
             constraints=JSONSerializer().serialize(
-                models.ConstraintModel.objects.filter(card__pk__in=[card_dict["cardid"] for card_dict in serialized_graph["cards"]])
+                models.ConstraintModel.objects.filter(
+                    card__pk__in=[
+                        card_dict["cardid"] for card_dict in serialized_graph["cards"]
+                    ]
+                )
             ),
         )
 
@@ -262,13 +355,14 @@ class GraphDesignerView(GraphBaseView):
         if not self.graph.isresource:
             help_title = _("Designing a Branch")
 
-        context["nav"]["help"] = {"title": help_title, "template": "graph-tab-help"}
+        context["nav"]["help"] = {"title": help_title, "templates": ["graph-tab-help"]}
 
         return render(request, "views/graph-designer.htm", context)
 
-class GraphDataView(View):
 
+class GraphDataView(View):
     action = "update_node"
+
     def get(self, request, graphid, nodeid=None):
         if self.action == "export_graph":
             graph = get_graphs_for_export([graphid])
@@ -276,7 +370,9 @@ class GraphDataView(View):
             graph_name = I18n_String(graph["graph"][0]["name"])
             f = JSONSerializer().serialize(graph, indent=4)
             response = HttpResponse(f, content_type="json/plain")
-            response["Content-Disposition"] = 'attachment; filename="%s.json"' % (graph_name)
+            response["Content-Disposition"] = 'attachment; filename="%s.json"' % (
+                graph_name
+            )
             return response
         elif self.action == "export_mapping_file":
             files_for_export = create_mapping_configuration_file(graphid, True)
@@ -295,7 +391,9 @@ class GraphDataView(View):
             buffer.close()
 
             response = HttpResponse()
-            response["Content-Disposition"] = "attachment; filename=" + str(file_name) + ".zip"
+            response["Content-Disposition"] = (
+                "attachment; filename=" + str(file_name) + ".zip"
+            )
             response["Content-length"] = str(len(zip_stream))
             response["Content-Type"] = "application/zip"
             response.write(zip_stream)
@@ -306,7 +404,12 @@ class GraphDataView(View):
             graph = Graph.objects.get(graphid=graphid)
             ret = graph.get_valid_domain_ontology_classes()
             for r in ret:
-                res.append({"ontology_property": r["ontology_property"], "ontology_classes": [c for c in r["ontology_classes"]]})
+                res.append(
+                    {
+                        "ontology_property": r["ontology_property"],
+                        "ontology_classes": [c for c in r["ontology_classes"]],
+                    }
+                )
             return JSONResponse(res)
 
         elif self.action == "get_nodes":
@@ -316,7 +419,9 @@ class GraphDataView(View):
         elif self.action == "get_related_nodes":
             parent_nodeid = request.GET.get("parent_nodeid", None)
             graph = Graph.objects.get(graphid=graphid)
-            ret = graph.get_valid_ontology_classes(nodeid=nodeid, parent_nodeid=parent_nodeid)
+            ret = graph.get_valid_ontology_classes(
+                nodeid=nodeid, parent_nodeid=parent_nodeid
+            )
             return JSONResponse(ret)
 
         elif self.action == "get_valid_domain_nodes":
@@ -335,7 +440,7 @@ class GraphDataView(View):
             if self.action == "import_graph":
                 graph_file = request.FILES.get("importedGraph").read()
                 graphs = JSONDeserializer().deserialize(graph_file)["graph"]
-                ret = GraphImporter.import_graph(graphs)
+                ret = GraphImporter.import_graph(graphs, user=request.user)
             else:
                 if graphid is not None:
                     graph = Graph.objects.get(graphid=graphid)
@@ -349,9 +454,32 @@ class GraphDataView(View):
 
                 elif self.action == "update_node":
                     old_node_data = graph.nodes.get(uuid.UUID(data["nodeid"]))
-                    nodegroup_changed = str(old_node_data.nodegroup_id) != data["nodegroup_id"]
+
+                    if (
+                        graph.isresource
+                        and old_node_data.datatype != "semantic"
+                        and old_node_data.datatype != data["datatype"]
+                    ):
+                        return JSONErrorResponse(
+                            _("Datatype Error"),
+                            _(
+                                """If you want to change the datatype of an existing node.
+                                Delete and then re-create the node, or export the branch then edit the datatype and re-import the branch."""
+                            ),
+                        )
+
+                    nodegroup_changed = (
+                        str(old_node_data.nodegroup_id) != data["nodegroup_id"]
+                    )
                     updated_values = graph.update_node(data)
                     if "nodeid" in data and nodegroup_changed is False:
+                        if not self.validate_images_only_config(old_node_data, data):
+                            return JSONErrorResponse(
+                                _("Datatype Error"),
+                                _(
+                                    "This node cannot be restricted to images only as it holds non-images already."
+                                ),
+                            )
                         graph.save(nodeid=data["nodeid"])
                     else:
                         graph.save()
@@ -384,21 +512,37 @@ class GraphDataView(View):
                     graph.save()
 
                 elif self.action == "move_node":
-                    ret = graph.move_node(data["nodeid"], data["property"], data["newparentnodeid"])
+                    ret = graph.move_node(
+                        data["nodeid"], data["property"], data["newparentnodeid"]
+                    )
                     graph.save()
 
                 elif self.action == "export_branch":
                     clone_data = graph.copy(root=data)
                     clone_data["copy"].slug = None
+                    clone_data["copy"].publication = None
+
                     clone_data["copy"].save()
+
+                    if bool(graph.publication_id):
+                        clone_data["copy"].publish(user=request.user)
+
                     ret = {"success": True, "graphid": clone_data["copy"].pk}
 
                 elif self.action == "clone_graph":
                     clone_data = graph.copy()
                     ret = clone_data["copy"]
                     ret.slug = None
+                    ret.publication = None
+
                     ret.save()
-                    ret.copy_functions(graph, [clone_data["nodes"], clone_data["nodegroups"]])
+
+                    if bool(graph.publication_id):
+                        ret.publish(user=request.user)
+
+                    ret.copy_functions(
+                        graph, [clone_data["nodes"], clone_data["nodegroups"]]
+                    )
 
                 elif self.action == "reorder_nodes":
                     json = request.body
@@ -411,23 +555,29 @@ class GraphDataView(View):
                                 for node in data["nodes"]:
                                     no = models.Node.objects.get(pk=node["nodeid"])
                                     no.sortorder = sortorder
-                                    no.save()
+                                    try:
+                                        if no.graph.publication_id == None:
+                                            no.save()
+                                        else:
+                                            ret = {
+                                                "title": _("Unable to Save Graph"),
+                                                "message": _(
+                                                    "You cannot edit a published graph.  Please unpublish the graph before editing."
+                                                ),
+                                            }
+                                            raise ValidationError(ret)
+                                    except ValidationError as e:
+                                        return JSONErrorResponse(
+                                            content=e.args[0], status=403
+                                        )
                                     sortorder = sortorder + 1
                             ret = data
 
-            return JSONResponse(ret)
+            return JSONResponse(ret, force_recalculation=True)
         except GraphValidationError as e:
             return JSONErrorResponse(e.title, e.message, {"status": "Failed"})
         except PublishedModelError as e:
             return JSONErrorResponse(e.title, e.message)
-        except RequestError as e:
-            return JSONErrorResponse(
-                _("Elasticsearch indexing error"),
-                _(
-                    """If you want to change the datatype of an existing node.
-                    Delete and then re-create the node, or export the branch then edit the datatype and re-import the branch."""
-                ),
-            )
 
     @method_decorator(group_required("Graph Editor"), name="dispatch")
     def delete(self, request, graphid):
@@ -435,6 +585,11 @@ class GraphDataView(View):
             data = JSONDeserializer().deserialize(request.body)
             try:
                 graph = Graph.objects.get(graphid=graphid)
+                if graph.publication:
+                    return JSONErrorResponse(
+                        _("Unable to delete nodes of a published graph"),
+                        _("Please unpublish your graph before deleting a node"),
+                    )
                 graph.delete_node(node=data.get("nodeid", None))
                 return JSONResponse({})
             except GraphValidationError as e:
@@ -442,12 +597,13 @@ class GraphDataView(View):
         elif self.action == "delete_instances":
             try:
                 graph = Graph.objects.get(graphid=graphid)
-                graph.delete_instances()
+                resp = graph.delete_instances(userid=request.user.id)
+                success = resp["success"]
                 return JSONResponse(
                     {
-                        "success": True,
-                        "message": "All the resources associated with the Model '{0}' have been successfully deleted.".format(graph.name),
-                        "title": "Resources Successfully Deleted.",
+                        "success": resp["success"],
+                        "message": resp["message"],
+                        "title": f"Resources {'Successfully' if success else 'Unsuccessfully'} Deleted from {graph.name}.",
                     }
                 )
             except GraphValidationError as e:
@@ -458,9 +614,7 @@ class GraphDataView(View):
             try:
                 graph = Graph.objects.get(graphid=graphid)
                 if graph.isresource:
-                    graph.delete_instances()
-                    graph.publication = None
-                    graph.save(validate=False)
+                    graph.delete_instances(userid=request.user.id)
                 graph.delete()
                 return JSONResponse({"success": True})
             except GraphValidationError as e:
@@ -468,6 +622,31 @@ class GraphDataView(View):
 
         return HttpResponseNotFound()
 
+    @staticmethod
+    def validate_images_only_config(old_node_data, new_node_data):
+        if (
+            old_node_data.config.get("imagesOnly", None) is False
+            and new_node_data["config"]["imagesOnly"]
+        ):
+            nodegroup_id = new_node_data["nodegroup_id"]
+            for file_type in (
+                models.TileModel.objects.filter(
+                    nodegroup_id=nodegroup_id,
+                    data__has_key=str(old_node_data.pk),
+                )
+                .annotate(
+                    file_data=Func(
+                        F(f"data__{old_node_data.pk}"),
+                        function="JSONB_ARRAY_ELEMENTS",
+                    )
+                )
+                .values_list(F("file_data__type"), flat=True)
+                .distinct()
+            ):
+                if not file_type.startswith("image/"):
+                    return False
+        return True
+
 
 class GraphPublicationView(View):
     action = None
@@ -489,9 +668,19 @@ class GraphPublicationView(View):
             elif self.action == "unpublish":
                 graph.unpublish()
         except Exception as e:
-            return JSONErrorResponse(e)
+            logger.exception(e)
+            return JSONErrorResponse(
+                _("Unable to process publication"),
+                _("Please contact your administrator if issue persists"),
+            )
 
-        return JSONResponse({"graph": graph, "title": "Success!", "message": "The graph has been successfully updated."})
+        return JSONResponse(
+            {
+                "graph": graph,
+                "title": "Success!",
+                "message": "The graph has been successfully updated.",
+            }
+        )
 
 
 @method_decorator(group_required("Graph Editor"), name="dispatch")
@@ -503,16 +692,22 @@ class CardView(GraphBaseView):
         if self.action == "update_card":
             if data:
                 card = Card(data)
-                card.save()
-                return JSONResponse(card)
+                try:
+                    card.save()
+                    return JSONResponse(card)
+                except ValidationError as e:
+                    return JSONErrorResponse(content=e.args[0], status=403)
 
         if self.action == "reorder_cards":
             if "cards" in data and len(data["cards"]) > 0:
                 with transaction.atomic():
                     for card_data in data["cards"]:
-                        card = models.CardModel.objects.get(pk=card_data["id"])
+                        card = Card.objects.get(pk=card_data["id"])
                         card.sortorder = card_data["sortorder"]
-                        card.save()
+                        try:
+                            card.save()
+                        except ValidationError as e:
+                            return JSONErrorResponse(content=e.args[0], status=403)
                 return JSONResponse(data["cards"])
 
         return HttpResponseNotFound()
@@ -534,16 +729,26 @@ class FunctionManagerView(GraphBaseView):
             context = self.get_context_data(
                 main_script="views/graph/function-manager",
                 functions=JSONSerializer().serialize(models.Function.objects.all()),
-                applied_functions=JSONSerializer().serialize(models.FunctionXGraph.objects.filter(graph=self.graph)),
-                function_templates=models.Function.objects.exclude(component__isnull=True),
+                applied_functions=JSONSerializer().serialize(
+                    models.FunctionXGraph.objects.filter(graph=self.graph)
+                ),
+                function_templates=models.Function.objects.exclude(
+                    component__isnull=True
+                ),
             )
 
             context["graphs"] = JSONSerializer().serialize(
-                models.GraphModel.objects.all().exclude(graphid=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID), exclude=["functions"]
+                models.GraphModel.objects.all().exclude(
+                    graphid=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID
+                ),
+                exclude=["functions"],
             )
             context["nav"]["title"] = self.graph.name
             context["nav"]["menu"] = True
-            context["nav"]["help"] = {"title": _("Managing Functions"), "template": "function-help"}
+            context["nav"]["help"] = {
+                "title": _("Managing Functions"),
+                "templates": ["function-help"],
+            }
 
             return render(request, "views/graph/function-manager.htm", context)
         else:
@@ -554,8 +759,16 @@ class FunctionManagerView(GraphBaseView):
         self.graph = Graph.objects.get(graphid=graphid)
         with transaction.atomic():
             for item in data:
-                functionXgraph, created = models.FunctionXGraph.objects.update_or_create(
-                    pk=item["id"], defaults={"function_id": item["function_id"], "graph_id": graphid, "config": item["config"]}
+                (
+                    functionXgraph,
+                    created,
+                ) = models.FunctionXGraph.objects.update_or_create(
+                    pk=item["id"],
+                    defaults={
+                        "function_id": item["function_id"],
+                        "graph_id": graphid,
+                        "config": item["config"],
+                    },
                 )
                 item["id"] = functionXgraph.pk
 
@@ -588,7 +801,9 @@ class PermissionDataView(View):
         if codename not in self.perm_cache:
             try:
                 self.perm_cache[codename] = Permission.objects.get(
-                    codename=codename, content_type__app_label="models", content_type__model="nodegroup"
+                    codename=codename,
+                    content_type__app_label="models",
+                    content_type__model="nodegroup",
                 )
                 return self.perm_cache[codename]
             except:
@@ -600,7 +815,14 @@ class PermissionDataView(View):
         if self.action == "get_permission_manager_data":
             identities = []
             for group in Group.objects.all():
-                identities.append({"name": group.name, "type": "group", "id": group.pk, "default_permissions": group.permissions.all()})
+                identities.append(
+                    {
+                        "name": group.name,
+                        "type": "group",
+                        "id": group.pk,
+                        "default_permissions": group.permissions.all(),
+                    }
+                )
             for user in User.objects.filter(is_superuser=False):
                 groups = []
                 default_perms = []
@@ -631,22 +853,33 @@ class PermissionDataView(View):
             identity = Group.objects.get(pk=identityId)
             for nodegroup_id in nodegroup_ids:
                 nodegroup = models.NodeGroup.objects.get(pk=nodegroup_id)
+                group_perm_strings = [
+                    perm.codename for perm in get_group_perms(identity, nodegroup)
+                ]
                 perms = [
-                    {"codename": codename, "name": self.get_perm_name(codename).name} for codename in get_group_perms(identity, nodegroup)
+                    {"codename": codename, "name": self.get_perm_name(codename).name}
+                    for codename in group_perm_strings
                 ]
                 ret.append({"perms": perms, "nodegroup_id": nodegroup_id})
         else:
             identity = User.objects.get(pk=identityId)
             for nodegroup_id in nodegroup_ids:
                 nodegroup = models.NodeGroup.objects.get(pk=nodegroup_id)
+                user_perm_strings = [
+                    perm.codename for perm in get_user_perms(identity, nodegroup)
+                ]
                 perms = [
-                    {"codename": codename, "name": self.get_perm_name(codename).name} for codename in get_user_perms(identity, nodegroup)
+                    {"codename": codename, "name": self.get_perm_name(codename).name}
+                    for codename in user_perm_strings
                 ]
 
                 # only get the group perms ("defaults") if no user defined object settings have been saved
                 if len(perms) == 0:
                     perms = [
-                        {"codename": codename, "name": self.get_perm_name(codename).name}
+                        {
+                            "codename": codename,
+                            "name": self.get_perm_name(codename).name,
+                        }
                         for codename in set(get_group_perms(identity, nodegroup))
                     ]
                 ret.append({"perms": perms, "nodegroup_id": nodegroup_id})
@@ -711,7 +944,9 @@ class NodegroupView(View):
             res = []
             if nodegroupid is not None:
                 nodegroup = models.NodeGroup.objects.get(nodegroupid=nodegroupid)
-                exportable = False if nodegroup.exportable is None else nodegroup.exportable
+                exportable = (
+                    False if nodegroup.exportable is None else nodegroup.exportable
+                )
                 res.append({"exportable": exportable})
                 return JSONResponse(res)
             else:
@@ -726,7 +961,9 @@ class NodegroupView(View):
         if self.action == "exportable" and nodegroupid is not None:
             exportable = json.loads(request.POST.get("exportable"))
 
-            nodegroup = models.NodeGroup.objects.select_for_update().filter(nodegroupid=nodegroupid)
+            nodegroup = models.NodeGroup.objects.select_for_update().filter(
+                nodegroupid=nodegroupid
+            )
             with transaction.atomic():
                 for ng in nodegroup:
                     ng.exportable = exportable
