@@ -36,6 +36,7 @@ def index(request):
             "app_title": settings.APP_TITLE,
             "copyright_text": settings.COPYRIGHT_TEXT,
             "copyright_year": settings.COPYRIGHT_YEAR,
+            "app_version": settings.APP_VERSION,
             "version": __version__,
             "show_language_swtich": settings.SHOW_LANGUAGE_SWITCH,
         },
@@ -61,11 +62,7 @@ def function_templates(request, template):
 def templates(request, template):
     if not template:
         template = request.GET.get("template")
-
-    try:
-        return render(request, template)
-    except Exception as e:
-        print(e)
+    return render(request, template)
 
 
 def help_templates(request):
