@@ -16,40 +16,55 @@ You should have received a copy of the GNU Affero General Public License
 along with this program. If not, see <http://www.gnu.org/licenses/>.
 """
 
-from base64 import b64decode
-from datetime import datetime
 import logging
 import os
-import json
-from django.contrib.auth import authenticate
+
 from django.contrib.gis.geos import GEOSGeometry
 from django.core.cache import cache
 from django.db import connection
-from django.http import HttpResponseNotFound
+from django.http import Http404
 from django.shortcuts import render
-from django.utils.translation import get_language, ugettext as _
-from arches.app.models import models
-from arches.app.models.concept import Concept
-from arches.app.models.system_settings import settings
+from django.utils.translation import gettext as _
+from arches.app.models.models import (
+    MapMarker,
+    GraphModel,
+    DDataType,
+    Widget,
+    ReportTemplate,
+    CardComponent,
+    Geocoder,
+    Node,
+    SearchExportHistory,
+)
+from arches.app.models.concept import Concept, get_preflabel_from_conceptid
 from arches.app.utils.response import JSONResponse, JSONErrorResponse
-from arches.app.datatypes.datatypes import DataTypeFactory
 from arches.app.utils.betterJSONSerializer import JSONSerializer, JSONDeserializer
 from arches.app.search.search_engine_factory import SearchEngineFactory
-from arches.app.search.elasticsearch_dsl_builder import Bool, Match, Query, Nested, Terms, MaxAgg, Aggregation
+from arches.app.search.elasticsearch_dsl_builder import (
+    Bool,
+    Match,
+    Query,
+    Term,
+    Terms,
+    MaxAgg,
+    Aggregation,
+)
 from arches.app.search.search_export import SearchResultsExporter
 from arches.app.search.time_wheel import TimeWheel
 from arches.app.search.components.base import SearchFilterFactory
-from arches.app.search.mappings import RESOURCES_INDEX
 from arches.app.views.base import MapBaseManagerView
-from arches.app.models.concept import get_preflabel_from_conceptid
-from arches.app.utils.permission_backend import get_nodegroups_by_perm, user_is_resource_reviewer
+from arches.app.utils import permission_backend
+from arches.app.utils.permission_backend import (
+    get_nodegroups_by_perm,
+    user_is_resource_reviewer,
+)
+from arches.app.utils.decorators import group_required
 import arches.app.utils.zip as zip_utils
 import arches.app.utils.task_management as task_management
 from arches.app.utils.data_management.resources.formats.htmlfile import HtmlWriter
 import arches.app.tasks as tasks
 from io import StringIO
 from tempfile import NamedTemporaryFile
-from openpyxl import Workbook
 from arches.app.models.system_settings import settings
 
 logger = logging.getLogger(__name__)
@@ -57,25 +72,26 @@ logger = logging.getLogger(__name__)
 
 class SearchView(MapBaseManagerView):
     def get(self, request):
-        map_layers = models.MapLayer.objects.all()
-        map_markers = models.MapMarker.objects.all()
-        map_sources = models.MapSource.objects.all()
+        map_markers = MapMarker.objects.all()
         resource_graphs = (
-            models.GraphModel.objects.exclude(pk=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID)
+            GraphModel.objects.exclude(pk=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID)
             .exclude(isresource=False)
             .exclude(publication=None)
         )
-        geocoding_providers = models.Geocoder.objects.all()
-        search_components = models.SearchComponent.objects.all()
-        datatypes = models.DDataType.objects.all()
-        widgets = models.Widget.objects.all()
-        templates = models.ReportTemplate.objects.all()
-        card_components = models.CardComponent.objects.all()
+        geocoding_providers = Geocoder.objects.all()
+        search_component_factory = SearchFilterFactory(request)
+        searchview_instance = search_component_factory.get_searchview_instance()
+        if not searchview_instance:
+            raise Http404(_("Search view instance not found"))
+        search_components = searchview_instance.get_searchview_filters()
+
+        datatypes = DDataType.objects.all()
+        widgets = Widget.objects.all()
+        templates = ReportTemplate.objects.all()
+        card_components = CardComponent.objects.all()
 
         context = self.get_context_data(
-            map_layers=map_layers,
             map_markers=map_markers,
-            map_sources=map_sources,
             geocoding_providers=geocoding_providers,
             search_components=search_components,
             widgets=widgets,
@@ -107,10 +123,12 @@ class SearchView(MapBaseManagerView):
         context["nav"]["search"] = False
         context["nav"]["help"] = {
             "title": _("Searching the Database"),
-            "template": "search-help",
+            "templates": ["search-help"],
         }
         context["celery_running"] = task_management.check_if_celery_available()
-        context["export_html_templates"] = HtmlWriter.get_graphids_with_export_template()
+        context["export_html_templates"] = (
+            HtmlWriter.get_graphids_with_export_template()
+        )
 
         return render(request, "views/search.htm", context)
 
@@ -130,31 +148,63 @@ def search_terms(request):
     se = SearchEngineFactory().create()
     searchString = request.GET.get("q", "")
     user_is_reviewer = user_is_resource_reviewer(request.user)
+    permitted_nodegroups = get_permitted_nodegroups(request.user)
 
     i = 0
-    ret = {}
-    for index in ["terms", "concepts"]:
+    ret = {"terms": [], "concepts": []}
+    if len(permitted_nodegroups) == 0:
+        return JSONResponse(ret)
+    for index in list(ret.keys()):
         query = Query(se, start=0, limit=0)
         boolquery = Bool()
 
         if lang != "*":
-            boolquery.must(Match(field="language", query=lang, type="phrase_prefix"))
+            boolquery.must(Term(field="language", term=lang))
 
-        boolquery.should(Match(field="value", query=searchString.lower(), type="phrase_prefix"))
-        boolquery.should(Match(field="value.folded", query=searchString.lower(), type="phrase_prefix"))
         boolquery.should(
-            Match(field="value.folded", query=searchString.lower(), fuzziness="AUTO", prefix_length=settings.SEARCH_TERM_SENSITIVITY)
+            Match(field="value", query=searchString.lower(), type="phrase_prefix")
+        )
+        boolquery.should(
+            Match(
+                field="value.folded", query=searchString.lower(), type="phrase_prefix"
+            )
+        )
+        boolquery.should(
+            Match(
+                field="value.folded",
+                query=searchString.lower(),
+                fuzziness="AUTO",
+                prefix_length=settings.SEARCH_TERM_SENSITIVITY,
+            )
+        )
+        boolquery.should(
+            Match(field="displayname.value", query=searchString, fuzziness=2, boost=2)
         )
 
-        if user_is_reviewer is False and index == "terms":
-            boolquery.filter(Terms(field="provisional", terms=["false"]))
+        if index == "terms":
+            boolquery.filter(
+                Terms(
+                    field="nodegroupid", terms=[str(ng) for ng in permitted_nodegroups]
+                )
+            )
+
+            if user_is_reviewer is False:
+                boolquery.filter(Terms(field="provisional", terms=["false"]))
 
         query.add_query(boolquery)
         base_agg = Aggregation(
-            name="value_agg", type="terms", field="value.raw", size=settings.SEARCH_DROPDOWN_LENGTH, order={"max_score": "desc"}
+            name="value_agg",
+            type="terms",
+            field="value.raw",
+            size=settings.SEARCH_DROPDOWN_LENGTH,
+            order={"max_score": "desc"},
+        )
+        nodegroupid_agg = Aggregation(
+            name="nodegroupid", type="terms", field="nodegroupid"
+        )
+        top_concept_agg = Aggregation(
+            name="top_concept", type="terms", field="top_concept"
         )
-        nodegroupid_agg = Aggregation(name="nodegroupid", type="terms", field="nodegroupid")
-        top_concept_agg = Aggregation(name="top_concept", type="terms", field="top_concept")
         conceptid_agg = Aggregation(name="conceptid", type="terms", field="conceptid")
         max_score_agg = MaxAgg(name="max_score", script="_score")
 
@@ -164,14 +214,15 @@ def search_terms(request):
         base_agg.add_aggregation(nodegroupid_agg)
         query.add_aggregation(base_agg)
 
-        ret[index] = []
         results = query.search(index=index)
         if results is not None:
             for result in results["aggregations"]["value_agg"]["buckets"]:
                 if len(result["top_concept"]["buckets"]) > 0:
                     for top_concept in result["top_concept"]["buckets"]:
                         top_concept_id = top_concept["key"]
-                        top_concept_label = get_preflabel_from_conceptid(top_concept["key"], lang)["value"]
+                        top_concept_label = get_preflabel_from_conceptid(
+                            top_concept["key"], lang=lang if lang != "*" else None
+                        )["value"]
                         for concept in top_concept["conceptid"]["buckets"]:
                             ret[index].append(
                                 {
@@ -193,6 +244,7 @@ def search_terms(request):
                             "id": i,
                             "text": result["key"],
                             "value": result["key"],
+                            "nodegroupid": result["nodegroupid"]["buckets"][0]["key"],
                         }
                     )
                     i = i + 1
@@ -204,15 +256,15 @@ def get_resource_model_label(result):
     if len(result["nodegroupid"]["buckets"]) > 0:
         for nodegroup in result["nodegroupid"]["buckets"]:
             nodegroup_id = nodegroup["key"]
-            node = models.Node.objects.get(nodeid=nodegroup_id)
+            node = Node.objects.get(nodeid=nodegroup_id)
             graph = node.graph
         return "{0} - {1}".format(graph.name, node.name)
     else:
         return ""
 
 
+@group_required("Resource Exporter")
 def export_results(request):
-
     total = int(request.GET.get("total", 0))
     format = request.GET.get("format", "tilecsv")
     report_link = request.GET.get("reportlink", False)
@@ -223,7 +275,9 @@ def export_results(request):
         download_limit = settings.SEARCH_EXPORT_IMMEDIATE_DOWNLOAD_THRESHOLD
 
     if total > download_limit and format != "geojson":
-        if (settings.RESTRICT_CELERY_EXPORT_FOR_ANONYMOUS_USER is True) and (request.user.username == "anonymous"):
+        if (settings.RESTRICT_CELERY_EXPORT_FOR_ANONYMOUS_USER is True) and (
+            request.user.username == "anonymous"
+        ):
             message = _(
                 "Your search exceeds the {download_limit} instance download limit.  \
                 Anonymous users cannot run an export exceeding this limit.  \
@@ -246,21 +300,29 @@ def export_results(request):
                 ).format(**locals())
                 return JSONResponse({"success": True, "message": message})
             else:
-                message = _("Your search exceeds the {download_limit} instance download limit. Please refine your search").format(
-                    **locals()
-                )
+                message = _(
+                    "Your search exceeds the {download_limit} instance download limit. Please refine your search"
+                ).format(**locals())
                 return JSONResponse({"success": False, "message": message})
 
     elif format == "tilexl":
         exporter = SearchResultsExporter(search_request=request)
         export_files, export_info = exporter.export(format, report_link)
         wb = export_files[0]["outputfile"]
-        with NamedTemporaryFile() as tmp:
-            wb.save(tmp.name)
-            tmp.seek(0)
-            stream = tmp.read()
-            export_files[0]["outputfile"] = tmp
-            return zip_utils.zip_response(export_files, zip_file_name=f"{settings.APP_NAME}_export.zip")
+        try:
+            with NamedTemporaryFile(delete=False) as tmp:
+                wb.save(tmp.name)
+                tmp.seek(0)
+                stream = tmp.read()
+                export_files[0]["outputfile"] = tmp
+                result = zip_utils.zip_response(
+                    export_files, zip_file_name=f"{settings.APP_NAME}_export.zip"
+                )
+        except OSError:
+            logger.error("Temp file could not be created.")
+            raise
+        os.unlink(tmp.name)
+        return result
     else:
         exporter = SearchResultsExporter(search_request=request)
         export_files, export_info = exporter.export(format, report_link)
@@ -274,16 +336,21 @@ def export_results(request):
             dest = StringIO()
             dest.write(message)
             export_files.append({"name": "error.txt", "outputfile": dest})
-        return zip_utils.zip_response(export_files, zip_file_name=f"{settings.APP_NAME}_export.zip")
+        return zip_utils.zip_response(
+            export_files, zip_file_name=f"{settings.APP_NAME}_export.zip"
+        )
 
 
-def append_instance_permission_filter_dsl(request, search_results_object):
+def append_instance_permission_filter_dsl(request, search_query_object):
     if request.user.is_superuser is False:
-        has_access = Bool()
-        terms = Terms(field="permissions.users_with_no_access", terms=[str(request.user.id)])
-        nested_term_filter = Nested(path="permissions", query=terms)
-        has_access.must_not(nested_term_filter)
-        search_results_object["query"].add_query(has_access)
+        query: Query = search_query_object.get("query", None)
+        if query:
+            inclusions = permission_backend.get_permission_inclusions()
+            for inclusion in inclusions:
+                query.include(inclusion)
+            query.add_query(
+                permission_backend.get_permission_search_filter(request.user)
+            )
 
 
 def get_dsl_from_search_string(request):
@@ -292,114 +359,40 @@ def get_dsl_from_search_string(request):
 
 
 def search_results(request, returnDsl=False):
-    for_export = request.GET.get("export")
-    pages = request.GET.get("pages", None)
-    total = int(request.GET.get("total", "0"))
-    resourceinstanceid = request.GET.get("id", None)
-    load_tiles = request.GET.get("tiles", False)
-    if load_tiles:
-        try:
-            load_tiles = json.loads(load_tiles)
-        except TypeError:
-            pass
-    se = SearchEngineFactory().create()
-    permitted_nodegroups = get_permitted_nodegroups(request.user)
-    include_provisional = get_provisional_type(request)
     search_filter_factory = SearchFilterFactory(request)
-    search_results_object = {"query": Query(se)}
+    searchview_component_instance = search_filter_factory.get_searchview_instance()
+    if not searchview_component_instance:
+        unavailable_searchview_name = search_filter_factory.get_searchview_name()
+        message = _("No search-view named {0}").format(unavailable_searchview_name)
+        return JSONErrorResponse(
+            _("Search Failed"),
+            message,
+            status=400,
+        )
 
     try:
-        for filter_type, querystring in list(request.GET.items()) + list(request.POST.items()) + [("search-results", "")]:
-            search_filter = search_filter_factory.get_filter(filter_type)
-            if search_filter:
-                search_filter.append_dsl(search_results_object, permitted_nodegroups, include_provisional)
-        append_instance_permission_filter_dsl(request, search_results_object)
-    except Exception as err:
-        logger.exception(err)
-        return JSONErrorResponse(message=err)
+        response_object, search_query_object = (
+            searchview_component_instance.handle_search_results_query(
+                search_filter_factory, returnDsl
+            )
+        )
+        if returnDsl:
+            return search_query_object.pop("query")
+        else:
+            return JSONResponse(content=response_object)
+    except Exception as e:
+        message = _("There was an error retrieving the search results")
+        try:
+            message = e.args[0].get("message", message)
+        except:
+            logger.exception("Error retrieving search results:")
+            logger.exception(e)
 
-    dsl = search_results_object.pop("query", None)
-    if returnDsl:
-        return dsl
-    dsl.include("graph_id")
-    dsl.include("root_ontology_class")
-    dsl.include("resourceinstanceid")
-    dsl.include("points")
-    dsl.include("permissions.users_without_read_perm")
-    dsl.include("permissions.users_without_edit_perm")
-    dsl.include("permissions.users_without_delete_perm")
-    dsl.include("permissions.users_with_no_access")
-    dsl.include("geometries")
-    dsl.include("displayname")
-    dsl.include("displaydescription")
-    dsl.include("map_popup")
-    dsl.include("provisional_resource")
-    if load_tiles:
-        dsl.include("tiles")
-    if for_export or pages:
-        results = dsl.search(index=RESOURCES_INDEX, scroll="1m")
-        scroll_id = results["_scroll_id"]
-        if not pages:
-            if total <= settings.SEARCH_EXPORT_LIMIT:
-                pages = (total // settings.SEARCH_RESULT_LIMIT) + 1
-            if total > settings.SEARCH_EXPORT_LIMIT:
-                pages = int(settings.SEARCH_EXPORT_LIMIT // settings.SEARCH_RESULT_LIMIT) - 1
-        for page in range(int(pages)):
-            results_scrolled = dsl.se.es.scroll(scroll_id=scroll_id, scroll="1m")
-            results["hits"]["hits"] += results_scrolled["hits"]["hits"]
-    else:
-        results = dsl.search(index=RESOURCES_INDEX, id=resourceinstanceid)
-
-    ret = {}
-    if results is not None:
-        if "hits" not in results:
-            if "docs" in results:
-                results = {"hits": {"hits": results["docs"]}}
-            else:
-                results = {"hits": {"hits": [results]}}
-
-        # allow filters to modify the results
-        for filter_type, querystring in list(request.GET.items()) + [("search-results", "")]:
-            search_filter = search_filter_factory.get_filter(filter_type)
-            if search_filter:
-                search_filter.post_search_hook(search_results_object, results, permitted_nodegroups)
-
-        def get_localized_descriptor(resource, descriptor_type, language_codes):
-            descriptor = resource["_source"][descriptor_type]
-            result = descriptor[0] if len(descriptor) > 0 else None
-            for language_code in language_codes:
-                for entry in descriptor:
-                    if entry["language"] == language_code and entry["value"] != "":
-                        return entry
-            return result
-
-        descriptor_types = ("displaydescription", "displayname")
-        active_and_default_language_codes = (get_language(), settings.LANGUAGE_CODE)
-
-        for resource in results["hits"]["hits"]:
-            for descriptor_type in descriptor_types:
-                descriptor = get_localized_descriptor(resource, descriptor_type, active_and_default_language_codes)
-                if descriptor:
-                    resource["_source"][descriptor_type] = descriptor["value"]
-                    if descriptor_type == "displayname":
-                        resource["_source"]["displayname_language"] = descriptor["language"]
-                else:
-                    resource["_source"][descriptor_type] = _("Undefined")
-
-        ret["results"] = results
-
-        for key, value in list(search_results_object.items()):
-            ret[key] = value
-
-        ret["reviewer"] = user_is_resource_reviewer(request.user)
-        ret["timestamp"] = datetime.now()
-        ret["total_results"] = dsl.count(index=RESOURCES_INDEX)
-        ret["userid"] = request.user.id
-        return JSONResponse(ret)
-
-    else:
-        ret = {"message": _("There was an error retrieving the search results")}
-        return JSONResponse(ret, status=500)
+        return JSONErrorResponse(
+            _("Search Failed"),
+            message,
+            status=500,
+        )
 
 
 def get_provisional_type(request):
@@ -410,7 +403,9 @@ def get_provisional_type(request):
     """
 
     result = False
-    provisional_filter = JSONDeserializer().deserialize(request.GET.get("provisional-filter", "[]"))
+    provisional_filter = JSONDeserializer().deserialize(
+        request.GET.get("provisional-filter", "[]")
+    )
     user_is_reviewer = user_is_resource_reviewer(request.user)
     if user_is_reviewer is not False:
         if len(provisional_filter) == 0:
@@ -432,17 +427,31 @@ def get_provisional_type(request):
 
 
 def get_permitted_nodegroups(user):
-    return [str(nodegroup.pk) for nodegroup in get_nodegroups_by_perm(user, "models.read_nodegroup")]
+    return get_nodegroups_by_perm(user, "models.read_nodegroup")
 
 
 def buffer(request):
     spatial_filter = JSONDeserializer().deserialize(
-        request.GET.get("filter", {"geometry": {"type": "", "coordinates": []}, "buffer": {"width": "0", "unit": "ft"}})
+        request.GET.get(
+            "filter",
+            {
+                "geometry": {"type": "", "coordinates": []},
+                "buffer": {"width": "0", "unit": "ft"},
+            },
+        )
     )
 
-    if spatial_filter["geometry"]["coordinates"] != "" and spatial_filter["geometry"]["type"] != "":
+    if (
+        spatial_filter["geometry"]["coordinates"] != ""
+        and spatial_filter["geometry"]["type"] != ""
+    ):
         return JSONResponse(
-            _buffer(spatial_filter["geometry"], spatial_filter["buffer"]["width"], spatial_filter["buffer"]["unit"]), geom_format="json"
+            _buffer(
+                spatial_filter["geometry"],
+                spatial_filter["buffer"]["width"],
+                spatial_filter["buffer"]["unit"],
+            ),
+            geom_format="json",
         )
 
     return JSONResponse()
@@ -466,7 +475,11 @@ def _buffer(geojson, width=0, unit="ft"):
                 """SELECT ST_TRANSFORM(
                     ST_BUFFER(ST_TRANSFORM(ST_SETSRID(%s::geometry, 4326), %s), %s),
                 4326)""",
-                (geom.hex.decode("utf-8"), settings.ANALYSIS_COORDINATE_SYSTEM_SRID, width),
+                (
+                    geom.hex.decode("utf-8"),
+                    settings.ANALYSIS_COORDINATE_SYSTEM_SRID,
+                    width,
+                ),
             )
             res = cursor.fetchone()
             geom = GEOSGeometry(res[0], srid=4326)
@@ -494,9 +507,11 @@ def get_export_file(request):
     user = request.user
     url = None
     if exportid is not None:
-        export = models.SearchExportHistory.objects.get(pk=exportid)
+        export = SearchExportHistory.objects.get(pk=exportid)
         try:
             url = export.downloadfile.url
             return JSONResponse({"message": _("Downloading"), "url": url}, indent=4)
         except ValueError:
-            return JSONResponse({"message": _("The requested file is no longer available")}, indent=4)
+            return JSONResponse(
+                {"message": _("The requested file is no longer available")}, indent=4
+            )
