@@ -30,11 +30,10 @@ function install_postgres {
     sudo -u postgres createdb training -T template_postgis
 }
 
-function install_yarn {
+function install_npm {
     sudo apt-get update -y
     sudo apt-get install nodejs-legacy -y
     sudo apt-get install npm -y
-    sudo npm install -g yarn
 }
 
 function main {
@@ -54,7 +53,7 @@ function main {
   pip install virtualenv==15.1.0
 
   install_postgres
-  install_yarn
+  install_npm
 }
 
 main
