@@ -75,6 +75,11 @@ class Command(BaseCommand):
         whitelisted into the settings_local.py template
 
         """
-        files = [{"src": "arches/app/templates/index.htm", "dst": "arches/install/arches-templates/project_name/templates/index.htm"}]
+        files = [
+            {
+                "src": "arches/app/templates/index.htm",
+                "dst": "arches/install/arches-templates/project_name/templates/index.htm",
+            }
+        ]
         for f in files:
             shutil.copyfile(f["src"], f["dst"])
